unit fMD_GetResults;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 8/24/04 10:49a $
*       Developer:    doma.user@domain.ext
*       Site:         Hines OIFO
*
*       Description:  Results retriever form
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Version 1.0 - Patch 4 (Misc fixes)/CPUser/fMD_GetResults.pas $
*
* $History: fMD_GetResults.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/13/04   Time: 11:53a
 * Created in $/CP Version 1.0 - Patch 4 (Misc fixes)/CPUser
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 12/11/02   Time: 2:20p
 * Created in $/CP Version 1.0 Patch 1/CPUser
 * Version t13 as released
 *
 *
*
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  ComCtrls,
  uMD_Common,
  uMD_Objects,
  StdCtrls,
  ImgList, System.ImageList;

type
  TfrmMD_GetResults = class(TForm)
    Panel1: TPanel;
    lvResults: TListView;
    btnCancel: TButton;
    btnOK: TButton;
    ilImages: TImageList;
    procedure lvResultsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure lvResultsColumnClick(Sender: TObject; Column: TListColumn);
    procedure lvResultsCompare(Sender: TObject; Item1, Item2: TListItem;
      Data: integer; var Compare: integer);
  private
    FSortColumn: integer;
    FSortDescending: Boolean;
  public
    { Public declarations }
  end;

function GetInstrumentResults(AStudy: TMDStudy; var Files: TStringList): Boolean;

implementation

function GetInstrumentResults(AStudy: TMDStudy; var Files: TStringList): Boolean;
var
  CPResults: TStringList;
  CPFiles: TStringList;
  FMDate: TFMDateTime;
  i: integer;
begin
  CPResults := TStringList.Create;
  CPFiles := TStringList.Create;
  Result := False;
  with TfrmMD_GetResults.Create(Application) do
    try
      CallVistaServer(MDBroker, RPC_UTILITIES, ['GETRSLT', MDPatient.IEN, AStudy.FieldValue('.04', True)], nil, CPResults);
      if CPResults.Count > 1 then
        begin
          for i := 1 to CPResults.Count - 1 do
            with lvResults.Items.Add do
              begin
                ImageIndex := -1;
                Caption := GetPiece(CPResults[i], '^', 2);
                SubItems.Add('');
                try
                  FMDate := StrToFloat(GetPiece(CPResults[i], '^', 4));
                  SubItems[0] := FMDateTimeToStr(FMDate);
                except
                  on E: EConvertError do
                    SubItems[0] := 'Invalid Date/Time';
                else
                  raise;
                end;
                SubItems.Add(GetPiece(CPResults[i], '^', 6));
              end;
          FSortColumn := 0;
          FSortDescending := False;
          lvResultsColumnClick(nil, lvResults.Columns[1]);
          Result := (ShowModal = mrOk);
          if Result then
            for i := 0 to lvResults.Items.Count - 1 do
              if lvResults.Items[i].Selected then
                Files.Add(lvResults.Items[i].SubItems[1]);
        end
      else
        MessageDlgS('No results found for this study.');
    finally
      FreeAndNil(CPResults);
      FreeAndNil(CPFiles);
      Free;
    end;
end;

{$R *.DFM}

procedure TfrmMD_GetResults.lvResultsSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  btnOk.Enabled := lvResults.SelCount > 0;
end;

procedure TfrmMD_GetResults.FormCreate(Sender: TObject);
begin
  RestoreWindow(Self);
end;

procedure TfrmMD_GetResults.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveWindow(Self);
end;

procedure TfrmMD_GetResults.lvResultsColumnClick(Sender: TObject; Column: TListColumn);
begin
  lvResults.Columns[FSortColumn].ImageIndex := -1;

  if Column.Index = FSortColumn then
    FSortDescending := (FSortDescending = False);
  FSortColumn := Column.Index;
  if FSortDescending then
    Column.ImageIndex := 0
  else
    Column.ImageIndex := 1;
  lvResults.AlphaSort;
end;

procedure TfrmMD_GetResults.lvResultsCompare(Sender: TObject; Item1, Item2: TListItem; Data: integer; var Compare: integer);
begin
  case FSortColumn of
    0: Compare := AnsiCompareStr(
        Item1.Caption,
        Item2.Caption);
    1:
      begin
        try
          if StrToDateTime(Item1.SubItems[0]) > StrToDateTime(Item2.SubItems[0]) then
            Compare := 1
          else
            Compare := -1;
        except
          on EConvertError do
            Compare := 0;
        end
      end;
  else
    Compare := AnsiCompareStr(
      Item1.SubItems[FSortColumn - 1],
      Item2.SubItems[FSortColumn - 1]);
  end;

  if not FSortDescending then
    if Compare = 1 then
      Compare := -1
    else
      if Compare = -1 then
        Compare := 1
      else
        Compare := 0;
end;

end.

