unit fMD_LookupPatient;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 8/24/04 10:49a $
*       Developer:    doma.user@domain.ext
*       Site:         Hines OIFO
*
*       Description:  Universal Patient Lookup
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Version 1.0 - Patch 4 (Misc fixes)/CPUser/fMD_LookupPatient.pas $
*
* $History: fMD_LookupPatient.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/13/04   Time: 11:54a
 * Created in $/CP Version 1.0 - Patch 4 (Misc fixes)/CPUser
 * 
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 10/06/03   Time: 11:05a
 * Updated in $/CP Version 1.0/CPUser
 * Final check-in for SQA version t22
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 12/20/02   Time: 2:23p
 * Updated in $/CP Version 1.0 Patch 1/CPUser
 * Release t15 finalized
 * Enhancements include:
 * 1. User information added to CPUser main form
 * 2. Consult type corrected on report display
 * 3. Min/Max window issues corrected in Results window
 * 4. Single character lookup issue resolved
 * 5. IP Port updated to allow 1000-99999
 * 6. Pt Confirmation changed to Titlecase for space concerns
 * 7. Changing dates on clinic lookup fixed
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 12/11/02   Time: 2:22p
 * Created in $/CP Version 1.0 Patch 1/CPUser
 * Version t13 as released
 *
 *
*
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  StdCtrls,
  ComCtrls,
  TRPCB,
  ActnList,
  uMD_Common,
  uMD_Objects, System.Actions;

type
  TMDLookupType = (ltPatient, ltTeam, ltClinic, ltWard);

  TfrmMD_LookupPatient = class(TForm)
    pnlBottom: TPanel;
    pnlOptions: TPanel;
    pnlPtList: TPanel;
    lvPatientList: TListView;
    Bevel4: TBevel;
    tmrLookup: TTimer;
    actLookupPatient: TActionList;
    actChangeLookupType: TAction;
    actClearListView: TAction;
    actClearComboBox: TAction;
    actResetTimer: TAction;
    actGetDefaultList: TAction;
    pnlSelectPatient: TPanel;
    lblLookup: TLabel;
    pnlClinicDate: TPanel;
    dtpClinicDate: TDateTimePicker;
    pnlSelectOption: TPanel;
    pnlOption: TPanel;
    rbtnPatient: TRadioButton;
    rbtnTeam: TRadioButton;
    rbtnClinic: TRadioButton;
    rbtnWard: TRadioButton;
    Bevel1: TBevel;
    Label1: TLabel;
    lblSelectOption: TLabel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Label3: TLabel;
    Panel1: TPanel;
    lbxOption: TListBox;
    Bevel6: TBevel;
    Bevel7: TBevel;
    Bevel8: TBevel;
    pnlLookupByPatient: TPanel;
    Label4: TLabel;
    edtLookup: TEdit;
    Bevel5: TBevel;
    edtOption: TEdit;
    Panel2: TPanel;
    btnOK: TButton;
    btnCancel: TButton;
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure tmrLookupTimer(Sender: TObject);
    procedure actChangeLookupTypeExecute(Sender: TObject);
    procedure actClearListViewExecute(Sender: TObject);
    procedure actClearComboBoxExecute(Sender: TObject);
    procedure actResetTimerExecute(Sender: TObject);
    procedure cbxLookupClick(Sender: TObject);
    procedure cbxLookupKeyPress(Sender: TObject; var Key: Char);
    procedure lvPatientListSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure edtOptionChange(Sender: TObject);
    procedure edtOptionKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure lbxOptionClick(Sender: TObject);
    procedure edtOptionKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lbxOptionDblClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure pnlLookupByPatientResize(Sender: TObject);
  private
    FClinics: TStringList;
    FWards: TStringList;
    FTeams: TStringList;
    FLookupType: TMDLookupType;
    //FBroker: TRPCBroker; No longer used DP
    FDFN: string;
    procedure LoadPatientList(var ListOfPatients: TStringList; ListHeader: string = 'Select patient:');
    procedure LoadClinics;
    procedure LoadWards;
    procedure LoadTeams;
    { Private declarations }
  public
    { Public declarations }
  end;

const
  LOOKUPCAPTION: array[TMDLookupType] of string = (
    'Select Patient:',
    'Select Team:',
    'Select Clinic:',
    'Select Ward:');

function LookupPatient(Broker: TRPCBroker; var DFN: string): Boolean;

var
  frmMD_LookupPatient: TfrmMD_LookupPatient;

implementation

{$R *.DFM}
{$WARN UNSAFE_CAST OFF}

function LookupPatient(Broker: TRPCBroker; var DFN: string): Boolean;
begin
  Application.ProcessMessages;
  if frmMD_LookupPatient = nil then
    frmMD_LookupPatient := TfrmMD_LookupPatient.Create(Application);

  with frmMD_LookupPatient do
    if ShowModal = mrOK then
      begin
        DFN := FDFN;
        Result := True;
      end
    else
      begin
        DFN := '';
        Result := False;
      end;
end;

procedure TfrmMD_LookupPatient.btnOKClick(Sender: TObject);
begin
  if lvPatientList.Selected <> nil then
    begin
      FDFN := TMDRecordID(lvPatientList.Selected.Data).IEN;
      ModalResult := mrOK;
    end;
end;

procedure TfrmMD_LookupPatient.btnCancelClick(Sender: TObject);
begin
  FDFN := '';
  ModalResult := mrCancel;
end;

procedure TfrmMD_LookupPatient.tmrLookupTimer(Sender: TObject);
var
  retList: TStringList;

  procedure LoadList(AType: string);
  begin
    retList.Clear;
    while RetList.Count < 20 do
      retList.Add(AType + '-' + IntToStr(RetList.Count + 1));
  end;

begin
  tmrLookup.Enabled := False;
  if Length(edtLookup.Text) > 2 then
    begin
      Screen.Cursor := crHourGlass;
      retList := TStringList.Create;
      try
        CallVistaServer(
          MDBroker,
          RPC_TMDRECORDID,
          ['PTLKUP', '2', '', '', edtLookup.Text],
          nil,
          retList);
        LoadPatientList(retList);
      finally
        FreeAndNil(retList);
      end;
      Screen.Cursor := crDefault;
    end;
end;

procedure TfrmMD_LookupPatient.actChangeLookupTypeExecute(Sender: TObject);
begin
  actClearListView.Execute;
  FLookupType := TMDLookupType(TComponent(Sender).Tag);
  pnlSelectOption.Visible := (FLookupType <> ltPatient);
  pnlLookupByPatient.Visible := (FLookupType = ltPatient);
  pnlClinicDate.Visible := (FLookupType = ltClinic);
  lblSelectOption.Caption := LOOKUPCAPTION[FLookupType];

  lbxOption.Items.Clear;
  edtOption.Text := '';

  case FLookupType of
    ltPatient:
      begin
        edtLookup.Text := '';
        if Visible then
          edtLookup.SetFocus;
      end;
    ltWard:
      begin
        if FWards.Count = 0 then
          LoadWards;
        lbxOption.Items.Clear;
        edtOption.Text := '';
        lbxOption.Items.Assign(FWards);
      end;
    ltTeam:
      begin
        if FTeams.Count = 0 then
          LoadTeams;
        lbxOption.Items.Clear;
        edtOption.Text := '';
        lbxOption.Items.Assign(FTeams);
      end;
    ltClinic:
      begin
        if FClinics.Count = 0 then
          LoadClinics;
        lbxOption.Items.Clear;
        edtOption.Text := '';
        lbxOption.Items.Assign(FClinics);
      end;
  end;

  {
  edtLookup.Enabled := True;
  edtLookup.Visible := True;
  edtLookup.Text := '';
  lblLookup.Enabled := True;
  cbxLookup.Enabled := True;
  cbxLookup.Text := '';

  if FLookupType <> ltClinic then
    begin
      edtLookup.Width := pnlSelectScreen.Width - (edtLookup.Left * 2);
      cbxLookup.Width := pnlSelectScreen.Width - (cbxLookup.Left * 2);
      lblClinicDate.Visible := False;
      dtpClinicDate.Visible := False;
    end
  else
    begin
      edtLookup.Width := dtpClinicDate.Left - (edtLookup.Left * 2);
      cbxLookup.Width := dtpClinicDate.Left - (edtLookup.Left * 2);
      lblClinicDate.Visible := True;
      dtpClinicDate.Visible := True;
    end;

  case FLookupType of
    ltPatient:
      begin
      end;
    ltTeam:
      begin
        edtLookup.Visible := False;
        cbxLookup.Visible := True;
        CallServer(MDBroker, RPC_UTILITIES, ['TEAMS'], nil, nil);
        for i := 1 to MDBroker.Results.Count - 1 do
          cbxLookup.Items.AddObject(
            GetPiece(MDBroker.Results[i], '^', 2),
            TMDRecordID.CreateFromHeader(MDBroker, MDBroker.Results[i]));
      end;
    ltClinic:
      begin
      end;
    ltWard:
      begin
        edtLookup.Visible := False;
        cbxLookup.Visible := True;
        CallServer(MDBroker, RPC_UTILITIES, ['WARDS'], nil, nil);
        for i := 1 to MDBroker.Results.Count - 1 do
          cbxLookup.Items.AddObject(
            GetPiece(MDBroker.Results[i], '^', 2),
            TMDRecordID.CreateFromHeader(MDBroker, MDBroker.Results[i]));
      end;
  end;
  lblLookup.Caption := 'Select ' + TRadioButton(Sender).Caption + ':';
  }
end;

procedure TfrmMD_LookupPatient.actClearListViewExecute(Sender: TObject);
begin
  lblLookup.Caption := 'Select patient:';
  with lvPatientList do
    begin
      Items.BeginUpdate;
      while Items.Count > 0 do
        begin
          if Items[0].Data <> nil then
            TMDRecordID(Items[0].Data).Free;
          Items[0].Data := nil;
          Items.Delete(0);
        end;
      Items.EndUpdate;
      Refresh;
    end;
end;

procedure TfrmMD_LookupPatient.actClearComboBoxExecute(Sender: TObject);
begin
  {
  with cbxLookup.Items do
    while Count > 0 do
      begin
        if Objects[0] <> nil then
          TMDRecordID(Objects[0]).Free;
        Objects[0] := nil;
        Delete(0);
      end;
  }
end;

procedure TfrmMD_LookupPatient.actResetTimerExecute(Sender: TObject);
begin
  actClearListView.Execute;
  tmrLookup.Enabled := False;
  tmrLookup.Enabled := (edtLookup.Text <> '');
  if Sender is TWinControl then
    TWinControl(Sender).SetFocus;
end;

procedure TfrmMD_LookupPatient.cbxLookupClick(Sender: TObject);
begin
  {
  if not cbxLookup.DroppedDown then
    begin
      Screen.Cursor := crDefault;
      actGetDefaultList.Execute;
    end;
  }
end;

procedure TfrmMD_LookupPatient.cbxLookupKeyPress(Sender: TObject; var Key: Char);
begin
  {
  if Key = #13 then
    cbxLookupClick(Sender);
  if Key = #27 then
    begin
      cbxLookup.Visible := False;
      edtLookup.Visible := True;
      edtLookup.SetFocus;
    end;
  }
end;

procedure TfrmMD_LookupPatient.LoadPatientList(var ListOfPatients: TStringList; ListHeader: string = 'Select patient:');
var
  i: integer;
begin
  actClearListView.Execute;
  if ListOfPatients.Count < 2 then
    MessageDlgS(GetPiece(ListOfPatients[0], '^', 2))
  else
    begin
      lblLookup.Caption := ListHeader;
      lvPatientList.Items.BeginUpdate;
      for i := 1 to ListOfPatients.Count - 1 do
        with lvPatientList.Items.Add do
          begin
            Caption := GetPiece(ListOfPatients[i], '^', 2);
            SubItems.Add(GetPiece(ListOfPatients[i], '^', 3));
            SubItems.Add(GetPiece(ListOfPatients[i], '^', 10));
            SubItems.Add(GetPiece(ListOfPatients[i], '^', 11));
            Data := TMDRecordID.CreateFromHeader(MDBroker, ListOfPatients[i]);
          end;
      lvPatientList.Items.EndUpdate;
      if lvPatientList.Items.Count > 0 then
        begin
          lvPatientList.Items[0].Selected := True;
          lvPatientList.SetFocus;
        end;
    end;
end;

procedure TfrmMD_LookupPatient.lvPatientListSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  btnOK.Enabled := Selected;
  btnOK.Default := Selected;
end;

procedure TfrmMD_LookupPatient.edtOptionChange(Sender: TObject);
var
  i: integer;
  Found: Boolean;
  LookFor: string;
begin
  Found := False;
  actClearListView.Execute;
  if edtOption.Text <> '' then
    begin
      i := 0;
      LookFor := LowerCase(edtOption.Text);
      while (i < lbxOption.Items.Count - 1) and (not Found) do
        begin
          if LowerCase(Copy(lbxOption.Items[i], 1, Length(LookFor))) = LookFor then
            begin
              lbxOption.ItemIndex := i;
              edtOption.Text := lbxOption.Items[i];
              edtOption.SelStart := Length(LookFor);
              edtOption.SelLength := Length(edtOption.Text);
              Found := True;
            end
          else
            inc(i);
        end;
    end;

  if not Found then
    lbxOption.ItemIndex := -1;
end;

procedure TfrmMD_LookupPatient.edtOptionKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  with edtOption do
    begin
      if (Key = VK_RETURN) and (lbxOption.ItemIndex > -1) then
        begin
          Key := 0;
          lbxOptionDblClick(Sender);
        end
      else
        if (Key = VK_BACK) and (SelLength > 0) then
          begin
            Text := Copy(Text, 1, SelStart - 1);
            Key := 0;
          end;
    end;
end;

procedure TfrmMD_LookupPatient.lbxOptionClick(Sender: TObject);
begin
  if lbxOption.ItemIndex > -1 then
    begin
      edtOption.Text := lbxOption.Items[lbxOption.Itemindex];
      edtOption.SelectAll;
      edtOption.SetFocus;
      lbxOptionDblClick(Sender);
    end;
end;

procedure TfrmMD_LookupPatient.edtOptionKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
    Key := #0;
end;

procedure TfrmMD_LookupPatient.FormCreate(Sender: TObject);
begin
  rbtnPatient.Checked := True;
  dtpClinicDate.DateTime := Now;
  FTeams := TStringList.Create;
  FWards := TStringList.Create;
  FClinics := TStringList.Create;
  RestoreWindow(Self);
end;

procedure TfrmMD_LookupPatient.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FTeams);
  FreeAndNil(FClinics);
  FreeAndNil(FWards);
end;

procedure TfrmMD_LookupPatient.LoadClinics;
var
  i: integer;
begin
  CallVistaServer(MDBroker, RPC_UTILITIES, ['CLINICS'], nil, FClinics);
  if FClinics.Count > 0 then
    begin
      FClinics.Delete(0); {Count}
      for i := 0 to FClinics.Count - 1 do
        begin
          FClinics.Objects[i] := TMDRecordID.CreateFromHeader(MDBroker, FClinics[i]);
          FClinics[i] := GetPiece(FClinics[i], '^', 2);
        end;
    end;
end;

procedure TfrmMD_LookupPatient.LoadTeams;
var
  i: integer;
begin
  CallVistaServer(MDBroker, RPC_UTILITIES, ['TEAMS'], nil, FTeams);
  if FTeams.Count > 0 then
    begin
      FTeams.Delete(0); {Count}
      for i := 0 to FTeams.Count - 1 do
        begin
          FTeams.Objects[i] := TMDRecordID.CreateFromHeader(MDBroker, FTeams[i]);
          FTeams[i] := GetPiece(FTeams[i], '^', 2);
        end;
    end;
end;

procedure TfrmMD_LookupPatient.LoadWards;
var
  i: integer;
begin
  CallVistaServer(MDBroker, RPC_UTILITIES, ['WARDS'], nil, FWards);
  if FWards.Count > 0 then
    begin
      FWards.Delete(0); {Count}
      for i := 0 to FWards.Count - 1 do
        begin
          FWards.Objects[i] := TMDRecordID.CreateFromHeader(MDBroker, FWards[i]);
          FWards[i] := GetPiece(FWards[i], '^', 2);
        end;
    end;
end;

procedure TfrmMD_LookupPatient.lbxOptionDblClick(Sender: TObject);
var
  PtList: TStringList;
  ApptDt: string;
begin
  if lbxOption.ItemIndex > -1 then
    begin
      PtList := TStringList.Create;
      try
        with TMDRecordID(lbxOption.Items.Objects[lbxOption.ItemIndex]) do
          case FLookupType of
            ltTeam:
              begin
                CallVistaServer(MDBroker, RPC_UTILITIES, ['TEAMPTS', IEN], nil, PtList);
                LoadPatientList(PtList, 'Select patients from team ''' + HeaderData[1] + ''':');
              end;
            ltWard:
              begin
                CallVistaServer(MDBroker, RPC_UTILITIES, ['WARDPTS', IEN], nil, PtList);
                LoadPatientList(PtList, 'Select patients from ward ''' + HeaderData[1] + ''':');
              end;
            ltClinic:
              begin
                ApptDt := FloatToStr(DateTimeToFMDT(Trunc(dtpClinicDate.Date)));
                CallVistaServer(MDBroker, RPC_UTILITIES, ['CLINICPT', IEN, ApptDt], nil, PtList);
                LoadPatientList(PtList, 'Select patients from clinic ''' + HeaderData[1] + ''':');
              end;
          end;
      finally
        FreeAndNil(PtList);
      end;
    end;
end;

procedure TfrmMD_LookupPatient.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveWindow(Self);
end;

procedure TfrmMD_LookupPatient.pnlLookupByPatientResize(Sender: TObject);
begin
  edtLookup.Width := pnlLookupByPatient.Width - (edtLookup.Left * 2);
end;

end.

