unit fMD_QuickCheckIn;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 8/24/04 10:50a $
*       Developer:    doma.user@domain.ext
*       Site:         Hines OIFO
*
*       Description:  Quick check-in form if Imaging has already done something
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Version 1.0 - Patch 4 (Misc fixes)/CPUser/fMD_QuickCheckIn.pas $
*
* $History: fMD_QuickCheckIn.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/13/04   Time: 11:58a
 * Created in $/CP Version 1.0 - Patch 4 (Misc fixes)/CPUser
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 12/11/02   Time: 2:26p
 * Created in $/CP Version 1.0 Patch 1/CPUser
 * Version t13 as released
 *
 *
*
================================================================================
}
interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ExtCtrls,
  StdCtrls,
  uMD_Common,
  uMD_Objects;

type
  TfrmMD_QuickCheckin = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Label1: TLabel;
    btnCheckIn: TButton;
    btnCancel: TButton;
    lbxInstruments: TListBox;
    Panel3: TPanel;
    rbtnInstrument: TRadioButton;
    rbtnNoInstrument: TRadioButton;
    Bevel2: TBevel;
    Bevel3: TBevel;
    procedure rbtnNoInstrumentClick(Sender: TObject);
    procedure rbtnInstrumentClick(Sender: TObject);
    procedure btnCheckInClick(Sender: TObject);
  private
    FStudy: TMDStudy;
    procedure LoadInstruments(CPDefinition: string);
    { Private declarations }
  public
    { Public declarations }
  end;

procedure QuickCheckIn(AStudy: TMDStudy);

implementation

{$R *.DFM}

procedure QuickCheckIn(AStudy: TMDStudy);
begin
  with TfrmMD_QuickCheckin.Create(Application) do
    try
      FStudy := AStudy;
      LoadInstruments(FStudy.FieldValue('.04', True));
      ShowModal;
    finally
      while lbxInstruments.Items.Count > 0 do
        begin
          TMDRecordID(lbxInstruments.Items.Objects[0]).Free;
          lbxInstruments.Items.Delete(0);
        end;
      free;
    end;
end;

{ TfrmMD_QuickCheckin }

procedure TfrmMD_QuickCheckin.LoadInstruments(CPDefinition: string);
var
  i: integer;
  rec: TMDRecordID;
begin
  CallVistaServer(MDBroker, RPC_TMDPROCEDURE, ['GETINST', CPDefinition], nil, nil);

  for i := 1 to MDBroker.Results.Count - 1 do
    if GetPiece(MDBroker.Results[i], '^', 3) = '1' then
      begin
        rec := TMDRecordID.CreateFromHeader(MDBroker, MDBroker.Results[i]);
        lbxInstruments.Items.AddObject(rec.HeaderData[1], rec);
      end;
  if lbxInstruments.Items.Count > 0 then
    begin
      rbtnNoInstrument.Checked := False;
      rbtnInstrument.Checked := True;
      rbtnInstrument.Enabled := True;
      lbxInstruments.Enabled := True;
      if lbxInstruments.Items.Count = 1 then
        lbxInstruments.ItemIndex := 0
      else
        lbxInstruments.ItemIndex := -1;
    end
  else
    begin
      rbtnInstrument.Checked := False;
      rbtnInstrument.Enabled := False;
      rbtnNoInstrument.Checked := True;
      lbxInstruments.Enabled := False;
    end;
end;

procedure TfrmMD_QuickCheckin.rbtnNoInstrumentClick(Sender: TObject);
begin
  lbxInstruments.ItemIndex := -1;
  lbxInstruments.Enabled := False;
end;

procedure TfrmMD_QuickCheckin.rbtnInstrumentClick(Sender: TObject);
begin
  lbxInstruments.Enabled := True;
end;

procedure TfrmMD_QuickCheckin.btnCheckInClick(Sender: TObject);
var
  SubmissionStr: string;
begin
  if (lbxInstruments.ItemIndex > -1) and rbtnInstrument.Checked then
    SubmissionStr :=
      FStudy.IENS +
      '^^^' +
      TMDRecordID(lbxInstruments.Items.Objects[lbxInstruments.ItemIndex]).IEN
  else
    SubmissionStr := FStudy.IENS;

  CallVistaServer(MDBroker, RPC_TMDPATIENT, ['CHKIN', MDPatient.DFN, SubmissionStr], nil, SubmissionStr);

  if StrToIntDef(GetPiece(SubmissionStr, '^', 1), -1) < 0 then
    MessageDlgS(MDBroker.Results.Text, mtError);

  ModalResult := mrOK;
end;

end.

