unit fMD_SelectPatient;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 8/24/04 10:50a $
*       Developer:    doma.user@domain.ext
*       Site:         Hines OIFO
*
*       Description:  Allows for a calling appliction to obtain a patient
*                     DFN either by lookup or directly and display a form
*                     showing patient identifiers and any messages that may
*                     be needed to be displayed to the user prior to selection.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Version 1.0 - Patch 4 (Misc fixes)/CPUser/fMD_SelectPatient.pas $
*
* $History: fMD_SelectPatient.pas $
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/13/04   Time: 11:59a
 * Created in $/CP Version 1.0 - Patch 4 (Misc fixes)/CPUser
 *
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 1/16/03    Time: 3:12p
 * Updated in $/CP Version 1.0 Patch 1/CPUser
 * Updated to include resource file for ptSelect.
 * Released as t16
 *
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 12/20/02   Time: 2:25p
 * Updated in $/CP Version 1.0 Patch 1/CPUser
 * Release t15 finalized
 * Enhancements include:
 * 1. User information added to CPUser main form
 * 2. Consult type corrected on report display
 * 3. Min/Max window issues corrected in Results window
 * 4. Single character lookup issue resolved
 * 5. IP Port updated to allow 1000-99999
 * 6. Pt Confirmation changed to Titlecase for space concerns
 * 7. Changing dates on clinic lookup fixed
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 12/11/02   Time: 2:28p
 * Created in $/CP Version 1.0 Patch 1/CPUser
 * Version t13 as released
 *
 *
 *
*
*
================================================================================
}

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  TRPCB;

function SelectPatient(Broker: TRPCBroker; DFN: string): Boolean;

type
  TMessageLevel = (mlError, mlInfo, mlWarning, mlSecurity);

type
  TPatient = class(TObject)
  private
    FDFN: string;
    FIdentifiers: TList;
    FMessages: TList;
    FSensitive: Boolean;
    procedure SetSensitive(NewVal: Boolean);
  public
    constructor Create;
    destructor Destroy; override;
  published
    property DFN: string
      read FDFN write FDFN;
    property Identifiers: TList
      read FIdentifiers write FIdentifiers;
    property Messages: TList
      read FMessages write FMessages;
    property Sensitive: Boolean
      read FSensitive write SetSensitive;
  end;

type
  TPtIdentifier = class(TObject)
  private
    FCaption: string;
    FDisplayNonSensitive: string;
    FDisplaySensitive: string;
    FDisplayLabel: TLabel;
  public
    constructor Create;
    destructor Destroy; override;
  published
    property Caption: string
      read FCaption write FCaption;
    property DisplayNonSensitive: string
      read FDisplayNonSensitive write FDisplayNonSensitive;
    property DisplaySensitive: string
      read FDisplaySensitive write FDisplaySensitive;
    property DisplayLabel: TLabel
      read FDisplayLabel write FDisplayLabel;
  end;

type
  TPtMessage = class(TObject)
  protected
    FLevel: TMessageLevel;
    FHeader: string;
    FText: TStringList;
    FPanel: TPanel;
  public
    constructor Create(Level: TMessageLevel; HeaderText: string);
    destructor Destroy; override;
    property MessageLevel: TMessageLevel
      read FLevel write FLevel;
    property Header: string
      read FHeader write FHeader;
    property Text: TStringList
      read FText write FText;
    property Panel: TPanel
      read FPanel write FPanel;
  end;

type
  TfrmMD_SelectPatient = class(TForm)
    btnCancel: TButton;
    btnOk: TButton;
    pnlPtInfo: TPanel;
    btnAgree: TButton;
    procedure btnAgreeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FBroker: TRPCBroker;
    procedure LoadPatient(Patient: TPatient);
    procedure LoadMessages(Patient: TPatient);
  end;

var
  CurrentPatient: TPatient;

implementation

uses
  uMD_Common;

{$R *.DFM}
{$R rMD_SelectPatient.res}
{$WARN UNSAFE_CAST OFF}

function SelectPatient(Broker: TRPCBroker; DFN: string): Boolean;
var
  i: integer;
  Patient: TPatient;
  PtID: TPtIdentifier;
  PtMsg: TPtMessage;
  PtResults: TStringList;
begin
  { Create Patient Object }
  Patient := TPatient.Create;
  try
    Patient.DFN := DFN;
    Patient.Sensitive := False;
    CurrentPatient := Patient;

    PtResults := TStringList.Create;
    try
      CallVistaServer(Broker, 'MD TMDPATIENT', ['SELECT', DFN], nil, PtResults);

      { Build the Patient Identifiers }
      i := 0;
      while i < PtResults.Count do
      begin
        if getPiece(PtResults[i], '^', 1) = '$$PTID' then
        begin
          PtID := TPtIdentifier.Create;
          PtID.Caption := TitleCase(getPiece(PtResults[i], '^', 2));
          PtID.DisplayNonSensitive := getPiece(PtResults[i], '^', 3);
          PtID.DisplaySensitive := getPiece(PtResults[i], '^', 4);
          if PtID.DisplaySensitive = '' then
            PtID.DisplaySensitive := PtID.DisplayNonSensitive;
          Patient.Identifiers.Add(PtID);
        end;
        inc(i);
      end;

      { Load any messages }
      i := 0;
      while i < PtResults.Count - 1 do
      begin
        if getPiece(PtResults[i], '^', 1) = '$$MSGHDR' then
        begin
          case StrToIntDef(getPiece(PtResults[i], '^', 2), -1) of
            0:
              PtMsg := TPtMessage.Create(mlError,
                getPiece(PtResults[i], '^', 3));
            1:
              PtMsg := TPtMessage.Create(mlInfo,
                getPiece(PtResults[i], '^', 3));
            2:
              PtMsg := TPtMessage.Create(mlWarning,
                getPiece(PtResults[i], '^', 3));
            3:
              begin
                PtMsg := TPtMessage.Create(mlSecurity,
                  getPiece(PtResults[i], '^', 3));
                Patient.Sensitive := True;
              end;
          else
            PtMsg := TPtMessage.Create(mlInfo, getPiece(PtResults[i], '^', 3));
          end;

          inc(i);

          while (i < PtResults.Count - 1) and
            (getPiece(PtResults[i], '^', 1) <> '$$MSGEND') do
          begin
            PtMsg.Text.Add(PtResults[i]);
            inc(i);
          end;
          Patient.Messages.Add(PtMsg);
        end;
        inc(i);
      end;

      with TfrmMD_SelectPatient.Create(Application) do
      begin
        LoadPatient(Patient);
        LoadMessages(Patient);
        Position := poScreenCenter;
        FBroker := Broker;
        showmodal;
        if ModalResult = mrOk then
          SelectPatient := True
        else
          SelectPatient := False;
      end;
    finally
      FreeandNil(PtResults);
    end;
  finally
    Patient.free;
  end;
end;

//* TPtMessage *//

constructor TPtMessage.Create(Level: TMessageLevel; HeaderText: string);
begin
  FText := TStringList.Create;
  FLevel := Level;
  FHeader := HeaderText;
end;

destructor TPtMessage.Destroy;
begin
  FText.Clear;
  FText.free;
end;

//* TPtIdentifier *//

constructor TPtIdentifier.Create;
begin
  inherited Create;
end;

destructor TPtIdentifier.Destroy;
begin
  inherited Destroy;
end;

//* TPatient *//

constructor TPatient.Create;
begin
  inherited Create;
  FIdentifiers := TList.Create;
  FMessages := TList.Create;
end;

destructor TPatient.Destroy;
begin

  while FIdentifiers.Count > 0 do
    begin
      TPtIdentifier(FIdentifiers[0]).free;
      FIdentifiers.Delete(0);
    end;
  FIdentifiers.free;

  while FMessages.Count > 0 do
    begin
      TPtMessage(FMessages[0]).free;
      FMessages.Delete(0);
    end;
  FMessages.free;

  inherited Destroy;

end;

procedure TPatient.SetSensitive(NewVal: Boolean);
var
  i: integer;
begin
  FSensitive := NewVal;
  for i := 0 to Self.FIdentifiers.Count - 1 do
    with TPtIdentifier(Self.FIdentifiers[i]) do
      case NewVal of
        True: DisplayLabel.Caption := DisplaySensitive;
        False: DisplayLabel.Caption := DisplayNonSensitive;
      end;
end;

//* TfrmPtSelect *//

procedure TfrmMD_SelectPatient.LoadPatient(Patient: TPatient);
var
  i: integer;
  PtId: TPtIdentifier;
begin
  for i := 0 to Patient.Identifiers.Count - 1 do
    begin

      PtId := TPtIdentifier(Patient.Identifiers[i]);

      with TLabel.Create(pnlPtInfo) do
        begin
          Parent := pnlPtInfo;
          Left := 15;
          Top := i * (Canvas.TextHeight('X') + 5) + 10;
          Caption := PtId.Caption + ':';
        end;
      PtId.DisplayLabel := TLabel.Create(pnlPtInfo);

      with PtId.DisplayLabel do
        begin
          Parent := pnlPtInfo;
          Left := 160;
          Top := i * (Canvas.TextHeight('X') + 5) + 10;
          Font.Style := [fsBold];
          if Patient.Sensitive then
            Caption := TitleCase(PtId.DisplaySensitive)
          else
            Caption := TitleCase(PtId.DisplayNonSensitive);
        end;

    end;

  pnlPtInfo.Height := Patient.Identifiers.Count * (Self.Canvas.TextHeight('X') + 8);

  Height := pnlPtInfo.Height + 70;
  Resize;
end;

procedure TfrmMD_SelectPatient.LoadMessages(Patient: TPatient);
var
  ErrorReturned: Boolean;
  msgBitMap: TBitMap;
  PtMsg: TPtMessage;
  i: integer;
begin
  ErrorReturned := False;
  msgBitMap := TBitMap.Create;
  try
    msgBitMap.Transparent := True;
    msgBitMap.TransparentMode := tmAuto;

    for i := 0 to Patient.Messages.Count - 1 do
    begin
      PtMsg := TPtMessage(Patient.Messages[i]);

      PtMsg.Panel := TPanel.Create(Self);

      with PtMsg.Panel do
      begin
        Caption := '';
        BevelInner := bvNone;
        BevelOuter := bvNone;
        Parent := Self;

        if i > 0 then
        begin
          Top := TPtMessage(Patient.Messages[i - 1]).Panel.Top +
            TPtMessage(Patient.Messages[i - 1]).Panel.Height;
        end
        else
          Top := pnlPtInfo.Top + pnlPtInfo.Height + 5;

        Left := 5;
        Width := Self.Width - (Self.BorderWidth * 2) - 5;
        Height := PtMsg.Text.Count * (Canvas.TextHeight('X') + 4);
        if Height < 40 then
          Height := 40;
      end;

      with TLabel.Create(PtMsg.Panel) do
      begin
        Parent := PtMsg.Panel;
        AutoSize := False;
        Top := 5;
        Left := 45;
        Width := PtMsg.Panel.Width - 5;
        WordWrap := True;
        Caption := PtMsg.Header;
        Font.Style := [fsBold];
        AutoSize := True;
      end;

      with TLabel.Create(PtMsg.Panel) do
      begin
        Parent := PtMsg.Panel;
        AutoSize := False;
        Top := 10 + Self.Canvas.TextHeight('X');
        Left := 45;
        Width := PtMsg.Panel.Width - 50;
        WordWrap := True;
        Caption := PtMsg.Text.Text;
        AutoSize := True;
        PtMsg.Panel.Height := Height + 15;
        Self.Height := PtMsg.Panel.Top + PtMsg.Panel.Height + 70;
        Self.Resize;
      end;

      with TImage.Create(Self) do
      begin
        Transparent := True;
        Parent := PtMsg.Panel;
        Left := 5;
        Top := 5;
        Height := 32;
        Width := 32;
        case PtMsg.MessageLevel of
          mlInfo:
            msgBitMap.LoadFromResourceName(HInstance, 'MLINFO');
          mlWarning:
            msgBitMap.LoadFromResourceName(HInstance, 'MLWARNING');
          mlSecurity:
            begin
              msgBitMap.LoadFromResourceName(HInstance, 'MLSECURITY');
              btnOk.Enabled := False;
              btnOk.TabStop := False;
              btnOk.Default := False;
              btnAgree.Visible := True;
              btnAgree.TabStop := True;
            end;
          mlError:
            begin
              msgBitMap.LoadFromResourceName(HInstance, 'MLERROR');
              btnOk.ModalResult := mrCancel;
              btnCancel.Visible := False;
              ErrorReturned := True;
            end;
        end;
        Picture.Bitmap := msgBitMap;
      end;
    end;

    if ErrorReturned then
    begin
      btnOk.ModalResult := mrCancel;
      btnOk.Visible := True;
      btnCancel.Visible := False;
      btnAgree.Visible := False;
    end;

    Caption := 'Confirm Patient Selection';
    if (Patient.Messages.Count > 0) and (ErrorReturned = False) then
      Caption := Caption + ' - Please read notifications carefully';
    if (Patient.Messages.Count > 0) and (ErrorReturned = True) then
      Caption := Caption + ' - Unable to select this record';

  finally
    FreeandNil(msgBitMap);
  end;
end;

procedure TfrmMD_SelectPatient.btnAgreeClick(Sender: TObject);
begin
  CallVistaServer(FBroker, RPC_TMDPATIENT, ['LOGSECURITY', CurrentPatient.DFN, 'RPCCALL^Clinical Procedure GUI v1'], nil);
  btnOk.Enabled := True;
  btnOk.Default := True;
  btnOk.TabStop := True;
  btnAgree.Enabled := False;
  btnAgree.Default := False;
  btnAgree.TabStop := False;
  btnAgree.Caption := 'Access has been logged';
  CurrentPatient.Sensitive := False;
  btnOk.SetFocus;
end;

procedure TfrmMD_SelectPatient.FormCreate(Sender: TObject);
begin
  SetHelpIDs(Self, HELPUSER_PTSELECT);
end;

end.
