unit mMD_Lookup;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Buttons,
  TRPCB,
  MFunStr,
  ExtCtrls;

type
  TfraMD_Lookup = class(TFrame)
    tmrLookup: TTimer;
    cbxValues: TComboBox;
    edtValue: TComboBox;
    procedure FrameEnter(Sender: TObject);
    procedure FrameExit(Sender: TObject);
    procedure tmrLookupTimer(Sender: TObject);
    procedure cbxValuesKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbxValuesClick(Sender: TObject);
    procedure edtValueKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbxValuesExit(Sender: TObject);
    procedure FrameResize(Sender: TObject);
    procedure edtValueExit(Sender: TObject);
  private
    FIEN: string;
    FDDNumber: string;
    FBroker: TRPCBroker;
    FChanged: Boolean;
    FPreviousValue: string;
    procedure Lookup(Broker: TRPCBroker; DD: string; var SelectedIEN: string);
    procedure SetTextValue;
    procedure SetDDNumber(const Value: string);
    procedure SetIEN(const Value: string);
    procedure SetBroker(const Value: TRPCBroker);
    function GetIENS: string;
    { Private declarations }
  public
    procedure InitFrame(ABroker: TRPCBroker; ADDNumber: string; AIEN: string = '');
  published
    property Broker: TRPCBroker read FBroker write SetBroker;
    property DDNumber: string read FDDNumber write SetDDNumber;
    property IEN: string read FIEN write SetIEN;
    property IENS: string read GetIENS;
    { Public declarations }
  end;

type
  TLookupRecord = class(TObject)
  private
    FValue: string;
    FIEN: string;
    FDD: string;
  public
    constructor Create;
    constructor CreateFromRPCString(RPCString: string);
    destructor Destroy; override;
  published
    property DD: string
      read FDD write FDD;
    property IEN: string
      read FIEN write FIEN;
    property Value: string
      read FValue write FValue;
  end;

implementation

uses
  uMD_Common;

{$R *.DFM}

{ TfraGMV_Lookup }

procedure TfraMD_Lookup.InitFrame(ABroker: TRPCBroker; ADDNumber: string; AIEN: string = '');
begin
  FBroker := ABroker;
  FDDNumber := ADDNumber;
  IEN := AIEN;
  SetTextValue;
end;

procedure TfraMD_Lookup.SetTextValue;
begin
  cbxValues.Visible := False;
  if (FIEN <> '') and (FDDNumber <> '') then
    begin
      CallServer(FBroker, RPC_TMDRECORDID, ['GETDATA', FDDNumber + '^' + IENS + '^.01'], nil, nil);
      edtValue.Text := FBroker.Results[0];
      FChanged := False;
      FPreviousValue := edtValue.Text;
    end
  else
    edtValue.Text := '';
end;

procedure TfraMD_Lookup.Lookup(Broker: TRPCBroker; DD: string; var SelectedIEN: string);
var
  tmpList: TStringList;
  i: integer;
begin
  tmpList := TStringList.Create;
  try
    CallServer(Broker, RPC_TMDRECORDID, ['LOOKUP', FDDNumber + '^' + edtValue.Text], nil, tmpList);
    if Piece(tmpList[0], '^', 1) = '-1' then
      begin
        MessageDlgS(Piece(tmpList[0], '^', 2), mtError, [mbok], 0);
        SetTextValue;
        edtValue.SelectAll;
        edtValue.SetFocus;
      end
    else if Piece(tmpList[0], '^', 1) = '0' then
      begin
        MessageDlgS('No entries found matching ''' + edtValue.Text + '''.', mtInformation, [mbok], 0);
        SetTextValue;
        edtValue.SetFocus;
        edtValue.SelectAll;
      end
    else if Piece(tmpList[0], '^', 1) = '1' then
      begin
        edtValue.Text := Piece(tmpList[1], '^', 2);
        FPreviousValue := edtValue.Text;
        FChanged := False;
        FIEN := Piece(Piece(tmpList[1], '^', 1), ';', 2);
        edtValue.SetFocus;
        edtValue.SelectAll;
      end
    else
      begin
        tmpList.Delete(0); {Contains the header count record}
        for i := 0 to tmpList.Count - 1 do
          begin
            tmpList.Objects[i] := TLookupRecord.CreateFromRPCString(tmpList[i]);
            tmpList[i] := Piece(tmpList[i], '^', 2);
          end;
        cbxValues.Items.Assign(tmpList);
        cbxValues.Top := edtValue.Top;
        cbxValues.Left := edtValue.Left;
        cbxValues.Width := edtValue.Width;
        cbxValues.DroppedDown := True;
        cbxValues.Visible := True;
        cbxValues.ItemIndex := 0;
        cbxValues.SetFocus;
      end;
  finally
    FreeAndNil(tmpList);
  end;
end;

procedure TfraMD_Lookup.SetDDNumber(const Value: string);
begin
  FDDNumber := Value;
end;

procedure TfraMD_Lookup.SetIEN(const Value: string);
begin
  FIEN := Value;
end;

procedure TfraMD_Lookup.SetBroker(const Value: TRPCBroker);
begin
  FBroker := Value;
end;

function TfraMD_Lookup.GetIENS: string;
begin
  Result := FIEN + ',';
end;

procedure TfraMD_Lookup.FrameEnter(Sender: TObject);
begin
  tmrLookup.Enabled := False;
  FChanged := False;
  edtValue.SetFocus;
end;

procedure TfraMD_Lookup.FrameExit(Sender: TObject);
begin
  tmrLookup.Enabled := False;
end;

procedure TfraMD_Lookup.tmrLookupTimer(Sender: TObject);
begin
  tmrLookup.Enabled := False;
  if edtValue.Text <> '' then
    Lookup(FBroker, FDDNumber, FIEN)
  else
    FIEN := '';
end;

procedure TfraMD_Lookup.cbxValuesKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_DOWN) and (cbxValues.ItemIndex < (cbxValues.Items.Count - 1)) then
    cbxValues.ItemIndex := cbxValues.ItemIndex + 1;
  if (Key = VK_UP) and (cbxValues.ItemIndex > 0) then
    cbxValues.ItemIndex := cbxValues.ItemIndex - 1;
  if (Key = VK_RETURN) and (cbxValues.ItemIndex > -1) then
    cbxValuesClick(Sender);
  if (Key = VK_ESCAPE) then
    begin
      SetTextValue;
      edtValue.SetFocus;
      edtValue.SelectAll;
    end;
  Key := 0;
end;

procedure TfraMD_Lookup.cbxValuesClick(Sender: TObject);
begin
  tmrLookup.Enabled := False;
  if cbxValues.ItemIndex > -1 then
    begin
      edtValue.Text := cbxValues.Items[cbxValues.ItemIndex];
      FIEN := TLookupRecord(cbxValues.Items.Objects[cbxValues.ItemIndex]).IEN;
    end
  else
    SetTextValue;
  cbxValues.Items.Clear;
  cbxValues.Visible := False;
  FPreviousValue := edtValue.Text;
  FChanged := False;
end;

procedure TfraMD_Lookup.edtValueKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  tmrLookup.Enabled := False;
  if (((Key = VK_DOWN) and (ssAlt in Shift)) or (Key = VK_RETURN)) and ((edtValue.Text <> '') and FChanged) then
    begin
      Key := 0;
      tmrLookupTimer(Sender);
    end;
  FChanged := (FPreviousValue <> edtValue.Text);
  tmrLookup.Enabled := FChanged;
end;

{ TLookupRecord }

constructor TLookupRecord.Create;
begin
  inherited;
end;

constructor TLookupRecord.CreateFromRPCString(RPCString: string);
begin
  inherited Create;
  FDD := Piece(RPCString, ';', 1);
  IEN := Piece(Piece(RPCString, ';', 2), '^', 1);
  FValue := Piece(RPCString, '^', 2);
end;

destructor TLookupRecord.Destroy;
begin
  inherited;
end;

procedure TfraMD_Lookup.cbxValuesExit(Sender: TObject);
begin
  cbxValues.Visible := False;
  edtValue.SetFocus;
end;

procedure TfraMD_Lookup.FrameResize(Sender: TObject);
begin
  edtValue.Top := 0;
  edtValue.Left := 0;
  edtValue.Width := Width;
end;

procedure TfraMD_Lookup.edtValueExit(Sender: TObject);
begin
  tmrLookup.Enabled := False;
  if FChanged then
    tmrLookupTimer(Sender);
end;

end.

