unit uMD_Common;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 2 $  $Modtime: 11/15/04 12:00p $
*       Developer:    doma.user@domain.ext
*       Site:         Hines OIFO
*
*       Description:  Common utilities and functions
*
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Version 1.0 - Patch 4 (Misc fixes)/Common/uMD_Common.pas $
*
* $History: uMD_Common.pas $
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 11/18/04   Time: 3:50p
 * Updated in $/CP Version 1.0 - Patch 4 (Misc fixes)/Common
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/13/04   Time: 11:20a
 * Created in $/CP Version 1.0 - Patch 4 (Misc fixes)/Common
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 5/09/03    Time: 2:48p
 * Updated in $/CP Version 1.0/Common
 * Added the following Date/Time Formatting Constants:
 * FMT_FMDATE = 'mm/dd/yyyy'
 * FMT_FMDATETIME = 'mm/dd/yyyy@hh:mm AM/PM'
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 12/10/02   Time: 2:37p
 * Created in $/CP Version 1.0 Patch 1/Common
 * Version t13 as released
 *
 *
 *
*
*
================================================================================
}

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Trpcb,
//djs//  SharedRPCBroker,
  uMD_Objects,
  uVerInfo,
  ClipBrd,
  ShellAPI,
  imglist,
  ComCtrls,
  Menus;

var
  MDUser: TMDUser;
  MDBroker: TRPCBroker;
  MDRecordID: TObject;
  MDPatient: TMDPatient;
  MDVersionInformation: TVerInfoRes;
  MDRunAsCPRSSlave: Boolean;
  MDInitialDFN: string = '';

type
  TBuffer = array[0..1048576] of byte;
  TCrc32Table = array[0..255] of Int64;
  TFMDateTime = Double;

  {Begin Delphi Code Generation from file 8989.51}

type
  TMDParameter = (
    xparAllowExternalAttachments,
    xparGateway,
    xparCrcBypass,
    xparCrcValues,
    xparDaysForInstrumentData,
    xparFileExtensions,
    xparHfsScratch,
    xparImagingXfer,
    xparOfflineMessage,
    xparOnline,
    xparUserSettings,
    xparVersionChk,
    xparWeblink);

type
  TMDParameterEntity = (
    peUser,
    peDivision,
    peSystem);

const
  MDPARAMETERENTITY: array[TMDParameterEntity] of string = (
    'USR', 'DIV', 'SYS');

const
  MDPARAMETERNAME: array[TMDParameter] of string = (
    'MD ALLOW EXTERNAL ATTACHMENTS',
    'MD GATEWAY',
    'MD CRC BYPASS',
    'MD CRC VALUES',
    'MD DAYS FOR INSTRUMENT DATA',
    'MD FILE EXTENSIONS',
    'MD HFS SCRATCH',
    'MD IMAGING XFER',
    'MD OFFLINE MESSAGE',
    'MD ONLINE',
    'MD USER DEFAULTS',
    'MD VERSION CHK',
    'MD WEBLINK');

const
  MDPARAMETERDESC: array[TMDParameter] of string = (
    'Allow non-instrument attachments',
    'Background Processor Parameters',
    'Bypass CRC Checking',
    'Clinical Procedures CRC Values',
    'Temporary instrument data life (Days)',
    'Allowable file types',
    'Scratch HFS Directory',
    'Imaging Network Share',
    'Offline message',
    'Clinical Procedure Online/Offline',
    'Individual user settings',
    'Active Clinical Procedures executables',
    'Clinical Procedures Home Page');

  {End of Delphi Code Generation}

const
  {Broker context}
  RPC_CREATECONTEXT_MGR = 'MD GUI MANAGER';
  RPC_CREATECONTEXT_USER = 'MD GUI USER';

  {Object specific RPC constants}
  RPC_TMDUSER = 'MD TMDUSER';
  RPC_TMDPROCEDURE = 'MD TMDPROCEDURE';
  RPC_TMDPARAMETER = 'MD TMDPARAMETER';
  RPC_TMDPATIENT = 'MD TMDPATIENT';
  RPC_TMDRECORDID = 'MD TMDRECORDID';
  RPC_TMDOUTPUT = 'MD TMDOUTPUT';
  RPC_TMDTRANSACTION = 'MD TMDTRANSACTION';
  RPC_UTILITIES = 'MD UTILITIES';
  RPC_GATEWAY = 'MD GATEWAY';

  {Boolean constants}
  BOOLEAN201: array[Boolean] of string = ('0', '1');
  BOOLEAN2TF: array[Boolean] of string = ('F', 'T');
  BOOLEAN2YN: array[Boolean] of string = ('N', 'Y');
  BOOLEAN2OO: array[Boolean] of string = ('Off', 'On');
  BOOLEAN01: array[0..1] of Boolean = (False, True);

  {Color Constants}
  COLOR_PTBACKGROUND: integer = $00EEC093;
  COLOR_PTFOREGROUND: integer = clBlack;

  {Formatting Constants}
  FMT_FMDATE = 'mm/dd/yyyy';
  FMT_FMDATETIME = 'mm/dd/yyyy@hh:mm AM/PM';

  {HelpContext ID's}
  HELPMGR_MAIN = 1;
  HELPMGR_EDITINST = 3;
  HELPMGR_EDITPARAM = 6;
  HELPMGR_EDITPROC = 5;
  HELPMGR_EDITUSERACCESS = 7;
  HELPMGR_LOOKUP = 8;
  HELPMGR_PRINTSETUP = 9;
  HELPMGR_INSTWIZARD = 2;
  HELPMGR_PROCWIZARD = 4;

  HELPUSER_MAIN = 1;
  HELPUSER_FILTER = 3;
  HELPUSER_PAGEDISPLAY = 5;
  HELPUSER_PROCESSERROR = 4;
  HELPUSER_PTSELECT = 6;
  HELPUSER_RESULTSWIZARD = 9;

function InString(S: string; SubStrs: array of string; CaseSensitive: Boolean = True): Boolean;
function LockParameters: Boolean;
procedure GetAllParameterValues(AParameter: TMDParameter; AEntity: TMDParameterEntity; var RetInList: TStringList);
function GetParameterValue(AParameter: TMDParameter; AEntity: TMDParameterEntity = peUser; AInstance: string = '1'): string;
procedure GetWPParameterValue(var RetLst: TStringList; AParameter: TMDParameter; AEntity: TMDParameterEntity = peUser; AInstance: string = '1');
procedure SetWPParameterValue(WPTxt: TStringList; AParameter: TMDParameter; AEntity: TMDPArameterEntity = peUser; AInstance: string = '1');
procedure SetParameterValue(AValue: string; AParameter: TMDParameter; AEntity: TMDParameterEntity = peUser; AInstance: string = '1');
procedure DeleteParameter(AParameter: TMDParameter; AEntity: TMDParameterEntity = peUser; AInstance: string = '1');
function UnLockParameters: Boolean;
function FMDTToTDateTime(FMDateTime: TFMDateTime): TDateTime;
function FMDateTimeToStr(FMDateTime: TFMDateTime): string;
function FMNow: TDateTime;
procedure DeCodeUNC(AUNC: string; var AServer, AShare, ADirectory, AFileName: string);
function EnCodeUNC(AServer, AShare, ADirectory, AFileName: string): string;
function DateTimeToFMDT(DateTime: TDateTime; ReturnSeconds: Boolean = True): TFMDateTime;
procedure DisplayCPHomePage;
procedure SelectAllListItems(ListView: TListView);
function TitleCase(Value: string): string;
function GetPiece(Value, Delimiter: string; StartPiece: integer; EndPiece: integer = 0): string;
function SetPiece(Value, Delimiter: string; Piece: integer; NewPiece: string): string;
procedure SaveWindow(Form: TForm);
procedure RestoreWindow(Form: TForm);
function CPFilesFilter: string;
procedure CheckCRC32;
procedure AddItemToListView(AListView: TListView; ACaption: string; ASubItems: array of string; APointer: Pointer = nil);
procedure SetHelpIDs(Frame: TFrame; HelpID: integer); overload;
procedure SetHelpIDs(Form: TForm; HelpID: integer); overload;
function CallVistaServer(Broker: TRPCBroker; RemoteProcedure: string; Parameters: array of string; MultList: TStringList; var RetString: string): Boolean; overload;
function CallVistaServer(Broker: TRPCBroker; RemoteProcedure: string; Parameters: array of string; MultList: TStringList; RetList: TStringList = nil): Boolean; overload;
function DebugMessage(const Msg: string): Boolean;
function MessageDlgS(const Msg: string; DlgType: TMsgDlgType = mtInformation; Buttons: TMsgDlgButtons = [mbOK]; HelpCtx: LongInt = 0): Word;
procedure AddListViewItem(aListView: TListView; ACaption: string; ASubItems: array of string; AData: Pointer = nil);

implementation

// Common functions and procedures

function InString(S: string; SubStrs: array of string; CaseSensitive: Boolean = True): Boolean;
var
  i: integer;
begin
  i := 0;
  Result := False;
  while (i <= High(SubStrs)) and (Result = False) do
    begin
      if CaseSensitive then
        begin
          if Pos(SubStrs[i], S) > 0 then
            Result := True
          else
            inc(i)
        end
      else
        begin
          if Pos(LowerCase(SubStrs[i]), LowerCase(S)) > 0 then
            Result := True
          else
            inc(i)
        end;
    end
end;

function LockParameters: Boolean;
begin
  if CallVistaServer(MDBroker, RPC_TMDPARAMETER, ['LOCK'], nil, nil) then
    Result := GetPiece(MDBroker.Results[0], '^', 1) = '1'
  else
    Result := False;
end;

function UnLockParameters: Boolean;
begin
  if CallVistaServer(MDBroker, RPC_TMDPARAMETER, ['UNLOCK'], nil, nil) then
    Result := GetPiece(MDBroker.Results[0], '^', 1) = '1'
  else
    Result := False;
end;

procedure GetAllParameterValues(AParameter: TMDParameter; AEntity: TMDParameterEntity; var RetInList: TStringList);
begin
  CallVistaServer(
    MDBroker,
    RPC_TMDPARAMETER,
    ['GETLST', MDPARAMETERENTITY[AEntity], MDPARAMETERNAME[AParameter]],
    nil, RetInList);
  RetInList.Delete(0); {Removes count}
end;

function GetParameterValue(AParameter: TMDParameter; AEntity: TMDParameterEntity = peUser; AInstance: string = '1'): string;
begin
  CallVistaServer(
    MDBroker,
    RPC_TMDPARAMETER,
    ['GETPAR', MDPARAMETERENTITY[AEntity], MDPARAMETERNAME[AParameter], AInstance],
    nil, Result);
end;

procedure GetWPParameterValue(var RetLst: TStringList; AParameter: TMDParameter; AEntity: TMDParameterEntity = peUser; AInstance: string = '1');
begin
  CallVistaServer(
    MDBroker,
    RPC_TMDPARAMETER,
    ['GETWP', MDPARAMETERENTITY[AEntity], MDPARAMETERNAME[AParameter], AInstance], nil, RetLst);
  RetLst.Delete(0); {Removes count}
end;

procedure SetWPParameterValue(WPTxt: TStringList; AParameter: TMDParameter; AEntity: TMDPArameterEntity = peUser; AInstance: string = '1');
begin
  if WPTxt.Text <> '' then
    CallVistaServer(
      MDBroker,
      RPC_TMDPARAMETER,
      ['SETWP', MDPARAMETERENTITY[AEntity], MDPARAMETERNAME[AParameter], AInstance], WPTxt, nil)
  else
    DeleteParameter(AParameter, AEntity, AInstance);
end;

procedure DeleteParameter(AParameter: TMDParameter; AEntity: TMDParameterEntity = peUser; AInstance: string = '1');
begin
  CallVistaServer(
    MDBroker,
    RPC_TMDPARAMETER,
    ['DELPAR', MDPARAMETERENTITY[AEntity], MDPARAMETERNAME[AParameter], AInstance], nil, nil);
end;

procedure SetParameterValue(AValue: string; AParameter: TMDParameter; AEntity: TMDParameterEntity = peUser; AInstance: string = '1');
begin
  CallVistaServer(
    MDBroker,
    RPC_TMDPARAMETER,
    ['SETPAR', MDPARAMETERENTITY[AEntity], MDPARAMETERNAME[AParameter], AInstance, AValue],
    nil, nil);
end;

function DebugMessage(const Msg: string): Boolean;
{declared separately for quick search and cleanup before release}
begin
  Result := MessageDlgS(Msg + #13 + #13 + 'Continue?',
    mtConfirmation, [mbYes, mbNo]) = mrYes;
end;

function MessageDlgS(const Msg: string; DlgType: TMsgDlgType = mtInformation; Buttons: TMsgDlgButtons = [mbOK]; HelpCtx: LongInt = 0): Word;
const
  Sounds: array[TMsgDlgType] of integer = (
    MB_ICONEXCLAMATION,
    MB_ICONHAND,
    MB_OK,
    MB_ICONQUESTION,
    MB_ICONASTERISK);
begin
  MessageBeep(Sounds[DlgType]);
  Result := MessageDlg(Msg, DlgType, Buttons, HelpCtx);
end;

procedure DeCodeUNC(AUNC: string; var AServer, AShare, ADirectory, AFileName: string);
begin
  AUNC := ExpandUNCFileName(AUNC);
  AServer := '';
  AShare := '';
  ADirectory := '';
  AFileName := '';
  if Copy(AUNC, 1, 2) = '\\' then
    begin
      AServer := GetPiece(AUNC, '\', 3);
      AShare := GetPiece(AUNC, '\', 4);
      AFileName := ExtractFileName(AUNC);
      ADirectory := Copy(AUNC, Length(ExtractFileDrive(AUNC)) + 1, Length(AUNC));
      ADirectory := Copy(ADirectory, 1, Length(ADirectory) - Length(AFileName));
    end;
end;

function EnCodeUNC(AServer, AShare, ADirectory, AFileName: string): string;
begin
  Result := '\\' + AServer + '\' + AShare + ADirectory + AFileName;
end;

function CallVistaServer(Broker: TRPCBroker; RemoteProcedure: string; Parameters: array of string; MultList: TStringList; var RetString: string): Boolean;
begin
  Result := CallVistaServer(Broker, RemoteProcedure, Parameters, MultList, nil);
  RetString := Broker.Results[0];
end;

function CallVistaServer(Broker: TRPCBroker; RemoteProcedure: string; Parameters: array of string; MultList: TStringList; RetList: TStringList = nil): Boolean;
var
  i, j: integer;
begin
  Broker.RemoteProcedure := RemoteProcedure;
  i := 0;
  if SizeOf(Parameters) > 0 then
    while i <= High(Parameters) do
      if (Copy(Parameters[i], 1, 1) = '@') and (Parameters[i] <> '@') then
        begin
          Broker.Param[i].Value := Copy(Parameters[i], 2, Length(Parameters[i]));
          Broker.Param[i].PType := Reference;
          inc(i);
        end
      else
        begin
          Broker.Param[i].Value := Parameters[i];
          Broker.Param[i].PType := Literal;
          inc(i);
        end;

  if MultList <> nil then
    begin
      for j := 0 to MultList.Count - 1 do
        Broker.Param[i + 1].Mult[IntToStr(j)] := MultList[j];
      Broker.Param[i + 1].PType := List;
    end;

  try
    if RetList = nil then
      Broker.Call
    else
      begin
        RetList.Clear;
        Broker.lstCall(RetList);
      end;
    Result := True;

  except
    on e: EBrokerError do
      begin
        MessageDlgS(
          'Error encountered retrieving VistA data.' + #13 +
          'Trapped by uMDCommon.CallServer' + #13 +
          'Server: ' + Broker.Server + #13 +
          'Listener port: ' + IntToStr(Broker.ListenerPort) + #13 +
          'Remote procedure: ' + Broker.RemoteProcedure + #13 +
          'Error is: ' + #13 +
          e.Message, mtError, [mbok], e.HelpContext);
        Broker.Results.Insert(0, '-1^' + e.Message);
        Result := False;
      end
  else
    raise;
  end;
end;

procedure SetHelpIDs(Frame: TFrame; HelpID: integer);
var
  ComponentID: integer;
begin
  {Set all HelpContext ID's prior to setting them on form create}
  Frame.HelpContext := HelpID;
  for ComponentID := 0 to Frame.ComponentCount - 1 do
    if Frame.Components[ComponentID] is TWinControl then
      TWinControl(Frame.Components[ComponentID]).HelpContext := HelpID;
end;

procedure SetHelpIDs(Form: TForm; HelpID: integer);
var
  ComponentID: integer;
begin
  {Set all HelpContext ID's prior to setting them on form create}
  Form.HelpContext := HelpID;
  for ComponentID := 0 to Form.ComponentCount - 1 do
    if Form.Components[ComponentID] is TWinControl then
      TWinControl(Form.Components[ComponentID]).HelpContext := HelpID;
end;

function GetPiece(Value, Delimiter: string; StartPiece: integer; EndPiece: integer = 0): string;
var
  i: integer;
begin
  if EndPiece > 0 then {Two parameter mode}
    begin
      i := StartPiece;
      while i < EndPiece do
        begin
          Result := Result + getPiece(Value, Delimiter, i) + Delimiter;
          inc(i);
        end;
      Result := Result + getPiece(Value, Delimiter, EndPiece);
    end
  else
    if StartPiece < 1 then
      Result := ''
    else
      if Pos(Delimiter, Value) < 1 then
        Result := Value
      else
        begin
          i := 1;
          while i < StartPiece do
            begin
              if Pos(Delimiter, Value) < 1 then
                Value := ''
              else
                Value := Copy(Value, Pos(Delimiter, Value) + 1, Length(Value));
              inc(i);
            end;

          if Value <> '' then
            if Pos(Delimiter, Value) > 0 then
              Value := Copy(Value, 0, Pos(Delimiter, Value) - 1);

          Result := Value;
        end;
end;

function SetPiece(Value, Delimiter: string; Piece: integer; NewPiece: string): string;
var
  i: integer;
begin
  with TStringList.Create do
    try
      Add('');
      while Length(Value) > 0 do
        begin
          if Pos(Delimiter, Value) > 0 then
            begin
              Add(Copy(Value, 0, Pos(Delimiter, Value) - 1));
              Value := Copy(Value, Pos(Delimiter, Value) + Length(Delimiter), Length(Value));
            end
          else
            begin
              Add(Value);
              Value := '';
            end;
        end;

      if Piece = 1 then
        Result := NewPiece
      else
        Result := Strings[1];

      i := 2;

      while i < Count do
        begin
          if i = Piece then
            Result := Result + Delimiter + NewPiece
          else
            Result := Result + Delimiter + Strings[i];
          inc(i);
        end;
    finally
      free;
    end;
end;

function FMNow: TDateTime;
{Converts a Delphi TDateTime to a TFMDateTime (Double)}
var
  Year, Month, Day, Hour, Min, Sec, MSec: Word;
begin
  DecodeDate(Now, Year, Month, Day);
  DecodeTime(Now, Hour, Min, Sec, MSec);
  Result := EncodeDate(Year, Month, Day) + EncodeTime(Hour, Min, 0, 0);
end;

function DateTimeToFMDT(DateTime: TDateTime; ReturnSeconds: Boolean = True): TFMDateTime;
{Converts a Delphi TDateTime to a TFMDateTime (Double)}
var
  Year, Month, Day, Hour, Min, Sec, MSec: Word;
begin
  DecodeDate(DateTime, Year, Month, Day);
  DecodeTime(DateTime, Hour, Min, Sec, MSec);
  if Frac(DateTime) <> 0 then
    begin
      Result :=
        (Year - 1700) * 10000 +
        Month * 100 +
        Day +
        (Hour * 0.01) +
        (Min * 0.0001);
      if ReturnSeconds then
        Result := Result +
          (Sec * 0.000001)
    end
  else
    Result :=
      (Year - 1700) * 10000 +
      Month * 100 +
      Day;
end;

function FMDTToTDateTime(FMDateTime: Double): TDateTime;
{Takes a fileman Date/Time as a Double and converts
it to a TDateTime}
begin
  try
    if Frac(FMDateTime) <> 0 then
      Result := EncodeDate(
        trunc(FMDateTime * 0.0001) + 1700,
        trunc(Frac(FMDateTime * 0.0001) * 100),
        trunc(Frac(FMDateTime * 0.01) * 100)
        ) +
        EncodeTime(
        trunc(Frac(FMDateTime + 0.0000001) * 100),
        trunc(Frac((FMDateTime + 0.0000001) * 100) * 100),
        trunc(Frac((FMDateTime + 0.0000001) * 10000) * 100),
        0
        )
    else
      Result := EncodeDate(
        trunc(FMDateTime * 0.0001) + 1700,
        trunc(Frac(FMDateTime * 0.0001) * 100),
        trunc(Frac(FMDateTime * 0.01) * 100)
        );
  except
    on E: EConvertError do
      Result := 0;
  else
    raise;
  end;
end;

function FMDateTimeToStr(FMDateTime: TFMDateTime): string;
{Takes a Fileman data type TFMDateTime (Double) and attempts
to convert that item to a string via Delphi DateTimeToStr.
This function fails on non precise fileman date/times}
begin
  try
    Result := DateTimeToStr(FMDTToTDateTime(FMDateTime));
  except
    on E: EConvertError do
      Result := 'Imprecise Date/Time';
  else
    raise;
  end;
end;

procedure DisplayCPHomePage;
{Displays the URL in the clients default browser}
var
  URL: PChar;
begin
  URL := PChar('http://' + GetParameterValue(xparWeblink, peSystem));
  if Length(URL) < 8 then
    MessageDlgS('No Clinical Procedures Web Address found', mtError)
  else
    ShellExecute(0, nil, URL, nil, nil, SW_NORMAL);
end;

procedure SelectAllListItems(ListView: TListView);
{Takes the passed in TListView and selects all items
in that TListView}
var
  i: integer;
begin
  for i := 0 to ListView.Items.Count - 1 do
    begin
      ListView.Items[i].Selected := True;
    end;
end;

function TitleCase(Value: string): string;
{Returns a string value in Title Case format}
begin
  Result := UpperCase(Copy(Value, 1, 1));
  while Length(Value) > 1 do
    begin
      if
        ((Copy(Value, 1, 1) >= 'A') and (Copy(Value, 1, 1) <= 'Z')) or
        ((Copy(Value, 1, 1) >= 'a') and (Copy(Value, 1, 1) <= 'z')) then
        Result := Result + LowerCase(Copy(Value, 2, 1))
      else
        Result := Result + Copy(Value, 2, 1);
      Value := Copy(Value, 2, Length(Value) - 1);
    end;
end;

procedure AddListViewItem(aListView: TListView; ACaption: string; ASubItems: array of string; AData: Pointer = nil);
var
  i: integer;
begin
  with aListView.Items.Add do
    begin
      Caption := ACaption;
      i := 0;
      while i <= High(ASubItems) do
        begin
          if i < (aListView.Columns.Count - 1) then
            SubItems.Add(ASubItems[i]);
          inc(i);
        end;
      Data := AData;
    end;
end;

function CPFilesFilter: string;
{Returns a string for placement in a TOpenDialog as the filter
property to restict user selection to only file types defined
in the parameter MD FILE EXTENSIONS.}
var
  FileTypes: TStringList;
begin
  FileTypes := TStringList.Create;
  try
    GetAllParameterValues(xparFileExtensions, peSystem, FileTypes);
    if FileTypes.Count > 0 then
      begin
        Result := 'Clinical Procedures Files|';
        while FileTypes.Count > 0 do
          begin
            Result := Result + '*' + GetPiece(FileTypes[0], '^', 1);
            FileTypes.Delete(0);
            if FileTypes.Count > 0 then
              Result := Result + ';';
          end
      end
    else
      Result := 'Clinical Procedures Files|*.*';
  finally
    FreeAndNil(FileTypes);
  end;
end;

procedure CheckCRC32;
var
  CRCValue: PChar;
begin
  with TOpenDialog.Create(Application) do
    try
      if Execute then
        begin
          CRCValue := PChar(GetFileCRC32(FileName));
          if MessageDlg(
            'Clinical Procedures CRC Value Calculation' + #13 + #13 +
            'FileName: ' + FileName + #13 +
            'CRC Value: ' + CRCValue + #13 + #13 +
            'Copy this value to the clipboard for pasting?',
            mtInformation,
            [mbYes, mbNo],
            0) = mrYes then
            ClipBoard.SetTextBuf(CRCValue);
        end
    finally
      free;
    end;
end;

procedure SaveWindow(Form: TForm);
var
  Settings: string;
  State: integer;
begin
  State := integer(Form.WindowState);
  if Form.WindowState = wsMaximized then
    Form.WindowState := wsNormal;
  Form.Visible := False;
  Application.ProcessMessages;
  Settings :=
    IntToStr(Screen.Width) + '|' +
    IntToStr(Screen.Height) + '|' +
    IntToStr(Form.Top) + '|' +
    IntToStr(Form.Left) + '|' +
    IntToStr(Form.Height) + '|' +
    IntToStr(Form.Width) + '|' +
    IntToStr(State);
  SetParameterValue(Settings, xparUserSettings, peUser, Form.ClassName);
end;

procedure RestoreWindow(Form: TForm);
var
  Settings: string;
begin
  Settings := GetParameterValue(xparUserSettings, peUser, Form.ClassName);

  if (StrToIntDef(GetPiece(Settings, '|', 1), 0) <> Screen.Width) or
    (StrToIntDef(GetPiece(Settings, '|', 2), 0) <> Screen.Height) then
    Form.Position := poScreenCenter
  else
    begin
      Form.Top := StrToIntDef(GetPiece(Settings, '|', 3), trunc(Screen.Height * 0.1));
      Form.Left := StrToIntDef(GetPiece(Settings, '|', 4), trunc(Screen.Width * 0.1));
      Form.Height := StrToIntDef(GetPiece(Settings, '|', 5), trunc(Screen.Height * 0.8));
      Form.Width := StrToIntDef(GetPiece(Settings, '|', 6), trunc(Screen.Width * 0.8));
      Form.WindowState := TWindowState(StrToIntDef(GetPiece(Settings, '|', 7), 0));
    end;
end;

procedure AddItemToListView(AListView: TListView; ACaption: string; ASubItems: array of string; APointer: Pointer = nil);
var
  i: integer;
begin
  with AListView.Items.Add do
    begin
      Caption := ACaption;
      for i := Low(ASubItems) to High(ASubItems) do
        SubItems.Add(ASubItems[i]);
      Data := APointer;
    end;
end;

end.

