unit uMD_Objects;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 2 $  $Modtime: 11/15/04 12:02p $
*       Developer:    doma.user@domain.ext
*       Site:         Hines OIFO
*
*       Description:  All CP Object Definitions,
*                     Properties and Methods.
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Version 1.0 - Patch 4 (Misc fixes)/Common/uMD_Objects.pas $
*
* $History: uMD_Objects.pas $
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 11/18/04   Time: 3:50p
 * Updated in $/CP Version 1.0 - Patch 4 (Misc fixes)/Common
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/13/04   Time: 11:20a
 * Created in $/CP Version 1.0 - Patch 4 (Misc fixes)/Common
 *
 * *****************  Version 4  *****************
 * User: Zzzzzzpetitd Date: 10/06/03   Time: 10:56a
 * Updated in $/CP Version 1.0/Common
 * Final check-in for SQA version t22
 *
 * *****************  Version 3  *****************
 * User: Zzzzzzpetitd Date: 4/14/03    Time: 11:02a
 * Updated in $/CP Version 1.0 Patch 1/Common
 * Code lock down prior to V1.0
 *
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 3/14/03    Time: 12:28p
 * Updated in $/CP Version 1.0 Patch 1/Common
 * T18 released to test sites
 *
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 12/10/02   Time: 2:40p
 * Created in $/CP Version 1.0 Patch 1/Common
 * Version t13 as released
 *
 *
 *
*
*
================================================================================
}

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  ComCtrls,
  Menus,
  Trpcb,
//djs//  SharedRPCBroker,
  ExtCtrls,
  StdCtrls,
  fMD_Splash,
  fMD_AboutDlg,
  XWBHash,
  FileCtrl,
  RPCConf1,
  Printers;

type
  TMDRecordID = class(TObject)
  private
    FBroker: TRPCBroker;
    FIEN: string;
    FDDNumber: string;
    FDescription: string;
    FSubRecordOf: TMDRecordID;
    FOtherInfo: string;
    FLockMessage: string;
    FHeaderData: array[0..20] of string;
    FHeader: string;
    function GetIENS: string;
    function GetFileName: string;
    function GetCaption: string;
    function GetHeaderData(Index: integer): string;
    procedure SetHeaderData(Index: integer; const Value: string);
  public
    constructor Create(Broker: TRPCBroker);
    constructor CreateFromHeader(Broker: TRPCBroker; Header: string);
    destructor Destroy; override;
    function Lock: Boolean;
    function UnLock: Boolean;
    function Delete: Boolean;
    function Rename(NewValue: string): Boolean;
    function FieldValue(Field: string; Internal: Boolean = False): string;
    function SetFieldValue(Field: string; Value: string): Boolean;
    procedure SaveChanges;
    function RecordExists: Boolean;
    procedure GetMultiple(MultDDNumber: string; var ReturnList: TStringList);
    procedure GetWPText(Field: string; var ReturnList: TStringList);
    procedure DisplayObject;
    property HeaderData[Index: integer]: string
    read GetHeaderData write SetHeaderData;
  published
    property Broker: TRPCBroker
      read FBroker;
    property IEN: string
      read FIEN write FIEN;
    property DDNumber: string
      read FDDNumber write FDDNumber;
    property Header: string
      read FHeader;
    property Caption: string
      read getCaption;
    property Description: string
      read FDescription write FDescription;
    property SubRecordOf: TMDRecordID
      read FSubRecordOf write FSubRecordOf;
    property OtherInfo: string
      read FOtherInfo write FOtherInfo;
    property IENS: string
      read GetIENS;
    property LockMessage: string
      read FLockMessage;
    property FileName: string
      read GetFileName;
  end;

type
  TMDUserSetting = (
    usToolBars,
    usLastReport);

const
  MDUSERSETTING: array[TMDUserSetting] of string = (
    'Use Toolbar',
    'Last Report');

type
  TMDUser = class(TObject)
  private
    FBroker: TRPCBroker;
    FDemoMode: Boolean;
    FByPassCRCCheck: Boolean;
    FDebugMode: Boolean;
    FDTime: integer;
    FDUZ: string;
    FSignOnDt: TDateTime;
    FName: string;
    FDomainIEN: string;
    FDomain: string;
    FDivisionIEN: string;
    FDivision: string;
    FCPManager: Boolean;
    FTitle: string;
    FConfirmAppClose: Boolean;
    FHelpFileDirectory: string;
    //FApplicationVersion: string; Replace in uVersionInfo DP
    FUCI: string;
    FVolume: string;
    FNode: string;
    function GetUsingSharedBroker: Boolean;
  public
    constructor Create(Broker: TRPCBroker);
    destructor Destroy; override;
    function SignOn(Context: string; BypassOffline: Boolean): Boolean;
    function CreateContext(Context: string): Boolean;
    function SignOff: Boolean;
    function CheckESig: Boolean;
    function SubmitToTaskman(RoutineReference: string): Boolean;
    function GetUserParameter(ASetting: TMDUserSetting): string;
    procedure SetUserParameter(ASetting: TMDUserSetting; const Value: string);
  published
    { Published declarations }
    property Broker: TRPCBroker
      read FBroker;
    property DemoMode: Boolean
      read FDemoMode default False;
    property DTime: integer
      read FDTime;
    property ByPassCRCCheck: Boolean
      read FByPassCRCCheck default False;
    property DebugMode: Boolean
      read FDebugMode default False;
    property DUZ: string
      read FDUZ;
    property SignOnDateTime: TDateTime
      read FSignOnDt;
    property Name: string
      read FName;
    property DomainIEN: string
      read FDomainIEN;
    property Domain: string
      read FDomain;
    property DivisionIEN: string
      read FDivisionIEN;
    property Division: string
      read FDivision;
    property CPManager: Boolean
      read FCPManager;
    property Title: string
      read FTitle;
    property ConfirmAppClose: Boolean
      read FConfirmAppClose write FConfirmAppClose;
    property HelpFileDirectory: string
      read FHelpFileDirectory;
    property UCI: string
      read FUCI;
    property Volume: string
      read FVolume;
    property Node: string
      read FNode;
    property UsingSharedBroker: Boolean
      read GetUsingSharedBroker;
  end;

type
  TMDVString = class(TObject)
  private
    FVString: string;
    FDateTime: TDateTime;
    FLocationIEN: string;
    FLocation: string;
    FStatus: string;
  public
    constructor Create(Header: string);
    destructor Destroy; override;
  published
    property VString: string
      read FVString;
    property DateTime: TDateTime
      read FDateTime;
    property LocationIEN: string
      read FLocationIEN;
    property Location: string
      read FLocation;
    property Status: string
      read FStatus;
  end;

type
  TMDOutput = class(TObject)
  private
    FRoutine: string;
    FTitle: string;
    FCompressed: Boolean;
    FBroker: TRPCBroker;
  public
    constructor Create(Broker: TRPCBroker);
    destructor Destroy; override;
    function Execute: Boolean;
  published
    property Routine: string
      read FRoutine write FRoutine;
    property Title: string
      read FTitle write FTitle;
    property Compressed: Boolean
      read FCompressed write FCompressed default False;
    property Broker: TRPCBroker
      read FBroker;
  end;

type
  TMDStudyStatus = (
    ssNew,
    ssSubmitted,
    ssError,
    ssComplete,
    ssReadyToComplete,
    ssPendingInstrumentData);

const
  MDSTUDYSTATUS: array[TMDStudyStatus] of string = (
    'New',
    'Submitted',
    'Error',
    'Complete',
    'Ready to Complete',
    'Pending Instrument Data');

type
  TMDStudy = class(TMDRecordID)
  private
    function GetDateCreated: string;
    function GetImageCount: integer;
    function GetInstrument: string;
    function GetProcedureName: string;
    function GetSortableDateCreated: Double;
    function GetStatus: string;
    function GetTreatingSpecialityIEN: string;
    function GetAutoSubmitToVistAImaging: Boolean;
    procedure SetImageCount(const Value: integer);
  public
    constructor Create(Broker: TRPCBroker);
    destructor Destroy; override;
    procedure UpdateStatus(ANewStatus: TMDStudyStatus);
    procedure DisplayErrors(ADisplay: TMemo);
    procedure AddFile(AFileName: string; AResultReport: string = '');
    procedure AddICDCodes(ACodes: string);
    procedure AddCPTCodes(ACodes: string);
    procedure AddPhysician(APhysician: string);
    procedure RemoveFile(AFileName: string);
    function SubmitToVistaImaging(var StatusMessage: string): Boolean;
  published
    property AutoSubmitToVistAImaging: Boolean
      read GetAutoSubmitToVistAImaging;
    property TreatingSpecialityIEN: string
      read GetTreatingSpecialityIEN;
    property ProcedureName: string
      read GetProcedureName;
    property SortableDateCreated: Double
      read GetSortableDateCreated;
    property DateCreated: string
      read GetDateCreated;
    property Status: string
      read GetStatus;
    property Instrument: string
      read GetInstrument;
    property ImageCount: integer
      read GetImageCount write SetImageCount;
  end;

type
  TMDPatient = class(TMDRecordID)
  private
    FStudies: TStringList;
    FDOB: string;
    FSSN: string;
    FWardRoom: string;
    FPatientName: string;
    FAge: string;
    FSex: string;
    function GetStudies(Index: integer): TMDStudy;
    function GetStudyCount: integer;
    procedure ClearStudyList;
    procedure SetDFN(const Value: string);
    function GetDFN: string;
    function GetDemographics: string;
  public
    constructor Create(Broker: TRPCBroker);
    destructor Destroy; override;
    procedure LoadStudies;
    function NewStudy: TMDStudy;
    property Studies[Index: integer]: TMDStudy read GetStudies;
  published
    //property Broker: TRPCBroker
    //  read FBroker;
    property DFN: string
      read GetDFN write SetDFN;
    property PatientName: string
      read FPatientName;
    property Age: string
      read FAge;
    property Sex: string
      read FSex;
    property DOB: string
      read FDOB;
    property WardRoom: string
      read FWardRoom;
    property SSN: string
      read FSSN;
    property StudyCount: integer
      read GetStudyCount;
    property Demographics: string
      read GetDemographics;
  end;

type
  TMDConsultRequest = class(TMDRecordID)
  private
    function GetCPDefIEN: string;
    function GetCPDefName: string;
    function GetDateTimeOrdered: string;
    function GetSortableDateTimeOrdered: Double;
    function GetStatus: string;
    function GetUrgency: string;
    function GetCheckInCount: integer;
  published
    property Status: string
      read GetStatus;
    property CPDefName: string
      read GetCPDefName;
    property CPDefIEN: string
      read GetCPDefIEN;
    property Urgency: string
      read GetUrgency;
    property DateTimeOrdered: string
      read GetDateTimeOrdered;
    property SortableDateTimeOrdered: Double
      read GetSortableDateTimeOrdered;
    property CheckInCount: integer
      read GetCheckInCount;
  end;

implementation

uses uMD_Common, U_HelpMGR;

// TMDRecordID

constructor TMDRecordID.Create(Broker: TRPCBroker);
begin
  inherited Create;
  FBroker := Broker;
end;

constructor TMDRecordID.CreateFromHeader(Broker: TRPCBroker; Header: string);
var
  i: integer;
begin
  inherited Create;
  FBroker := Broker;

  FHeader := Header;
  for i := 0 to 20 do
    FHeaderData[i] := getPiece(Header, '^', i + 1);
  FDDNumber := GetPiece(FHeaderData[0], ';', 1);
  FIEN := GetPiece(FHeaderData[0], ';', 2);
end;

destructor TMDRecordID.Destroy;
begin
  FBroker := nil;
  inherited Destroy;
end;

procedure TMDRecordID.DisplayObject;
begin
  MessageDlg(
    ClassName + ' ' +
    'Object Display' + #13 + #13 +
    'FDDNumber:' + #9 + FDDNumber + #13 +
    'FIEN:' + #9 + #9 + FIEN + #13 +
    'IENS:' + #9 + #9 + GetIENS + #13 +
    'Caption:' + #9 + #9 + GetCaption + #13 +
    'FDescription:' + #9 + FDescription,
    mtInformation,
    [mbOk],
    0);
end;

function TMDRecordID.GETIENS: string;
var
  RecordID: TMDRecordID;
begin
  if FIEN <> '' then
    begin
      RecordID := Self;
      Result := RecordID.IEN + ',';
      while RecordID.SubRecordOf <> nil do
        begin
          RecordID := RecordID.SubRecordOf;
          Result := Result + RecordID.IEN + ',';
        end;
    end
  else
    Result := '';
end;

function TMDRecordID.GetFileName: string;
begin
  if CallVistaServer(FBroker, RPC_TMDRECORDID, ['FILENAME', FDDNumber], nil) then
    Result := GetPiece(FBroker.Results[0], '^', 2)
  else
    Result := 'UNKNOWN FILE';
end;

function TMDRecordID.GetCaption: string;
begin
  Result := FieldValue('.01');
end;

function TMDRecordID.Lock: Boolean;
begin
  if CallVistaServer(FBroker, RPC_TMDRECORDID, ['LOCK', FDDNumber, GetIENS], nil) then
    case StrToIntDef(GetPiece(FBroker.Results[0], '^', 1), 0) of
      1:
        begin
          Result := True;
        end;
    else
      begin
        Result := False;
        FBroker.Results.Delete(0);
        FLockMessage := FBroker.Results.Text;
      end;
    end
  else
    Result := False;
end;

function TMDRecordID.UnLock: Boolean;
begin
  if CallVistaServer(FBroker, RPC_TMDRECORDID, ['UNLOCK', FDDNumber, GetIENS], nil) then
    case StrToIntDef(GetPiece(FBroker.Results[0], '^', 1), 0) of
      1: Result := True;
    else
      Result := False;
    end
  else
    Result := False;
end;

function TMDRecordID.Rename(NewValue: string): Boolean;
begin
  Result := CallVistaServer(FBroker, RPC_TMDRECORDID, ['RENAME', FDDNumber, GetIENS, '', NewValue], nil);
end;

function TMDRecordID.Delete: Boolean;
begin
  Result := CallVistaServer(FBroker, RPC_TMDRECORDID, ['DELREC', FDDNumber, GetIENS], nil);
end;

function TMDRecordID.FieldValue(Field: string; Internal: Boolean = False): string;
begin
  CallVistaServer(MDBroker, RPC_TMDRECORDID, ['GETDATA', FDDNumber, GetIENS, Field, BOOLEAN201[Internal]], nil, Result);
end;

function TMDRecordID.RecordExists: Boolean;
begin
  if CallVistaServer(FBroker, RPC_TMDRECORDID, ['EXISTS', FDDNumber, IENS], nil, nil) then
    Result := (FBroker.Results[0] = '1')
  else
    Result := False;
end;

procedure TMDRecordID.GetMultiple(MultDDNumber: string; var ReturnList: TStringList);
begin
  ReturnList.Clear;
  CallVistaServer(FBroker, RPC_TMDRECORDID, ['GETLST', MultDDNumber, ',' + IENS, '.01'], nil, ReturnList);
end;

procedure TMDRecordID.GetWPText(Field: string; var ReturnList: TStringList);
begin
  ReturnList.Clear;
  CallVistaServer(FBroker, RPC_TMDRECORDID, ['GETDATA', FDDNumber, IENS, Field], nil, ReturnList);
  if ReturnList.Count > 0 then
    ReturnList.Delete(0); {Remove counter}
end;

// TMDTUser

constructor TMDUser.Create(Broker: TRPCBroker);
var
  i: integer;
begin
  inherited Create;
  FBroker := Broker;
  FDUZ := '';
  FName := '';
  FTitle := '';
  FDivisionIEN := '';
  FDivision := '';
  FDebugMode := False;
  for i := 1 to ParamCount do
    begin
      if uppercase(ParamStr(i)) = '/DEBUG' then
        FDebugMode := True;
    end;
end;

destructor TMDUser.Destroy;
begin
  FBroker := nil;
  inherited Destroy;
end;

function TMDUser.SignOn(Context: string; BypassOffline: Boolean): Boolean;
var
  i: integer;
  UseServerList: Boolean;
  SLServer, SLPort: string;
  CRCActual, CRCExpected: string;
  //f: textfile;

  function Online: Boolean;
    {Determine if the application is on-line after the user has signed on}
  var
    OffLineMsg: TStringList;
  begin
    if GetParameterValue(xparOnline, peSystem) <> 'YES' then
      if BypassOffline then
        begin
          Result := MessageDlgS(
            'Notice: Clinical Procedures is Marked ''Offline''.' + #13 + #13 +
            'Do you wish to proceed?', mtConfirmation, [mbYes, mbNo], 0) = mrYes;
        end
      else
        try
          OffLineMsg := TStringList.Create;
          GetWPParameterValue(OfflineMsg, xparOfflineMessage, peSystem);
          if Length(OffLineMsg.Text) < 3 then
            begin
              OffLineMsg.Clear;
              OffLineMsg.Add('Clinical Procedures is marked ''Offline'' at this time');
            end;
          MessageDlgS(OffLineMsg.Text);
          Result := False;
        finally
          FreeAndNil(OffLineMsg);
        end
    else
      Result := True;
  end;

begin
  Result := False;
  UseServerList := True;
  FHelpFileDirectory := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName)) + 'Help\';
  FBroker.Server := 'BROKERSERVER'; {VA Standard server name}
  FBroker.ListenerPort := 9200; {VA Standard listener port}
  FBroker.ClearParameters := True;
  FBroker.ClearResults := True;
  FDemoMode := False;

  if FBroker.Socket > 0 then
    begin
      Result := True;
      exit;
    end;

  for i := 1 to ParamCount do
    begin
      (*
      if InString(ParamStr(i), ['/demo', '-demo'], False) then
        begin
          FDemoMode := True;
          UseServerList := False;
        end;
      *)
      if InString(ParamStr(i), ['/s=', '-s=', 's=', '/server=', '-server=', 'server='], False) then
        begin
          FBroker.Server := GetPiece(ParamStr(i), '=', 2);
          UseServerList := False;
        end;

      if InString(ParamStr(i), ['/p=', '-p=', 'p=', '/port=', '-port=', 'port='], False) then
        begin
          FBroker.ListenerPort := StrToIntDef(GetPiece(ParamStr(i), '=', 2), 9200);
          UseServerList := False;
        end;

      if InString(ParamStr(i), ['/cprs', '-cprs', 'cprs'], False) then
        MDRunAsCPRSSlave := True;

      if InString(ParamStr(i), ['/dfn=', '-dfn=', 'dfn='], False) then
        MDInitialDFN := GetPiece(ParamStr(i), '=', 2);

      if InString(ParamStr(i), ['/helpdir=', '-helpdir=', 'helpdir='], False) then
        FHelpFileDirectory := ExcludeTrailingPathDelimiter(GetPiece(ParamStr(i), '=', 2)) + '\';

      if InString(ParamStr(i), ['/debug=', '-debug=', 'debug='], False) then
        begin
          FDebugMode := (uppercase(GetPiece(ParamStr(i), '=', 2)) = 'ON');
          FBroker.DebugMode := (uppercase(GetPiece(ParamStr(i), '=', 2)) = 'ON');
        end;

      if InString(ParamStr(i), ['/bypasscrc=', '-bypasscrc=', 'bypasscrc='], False) then
        FByPassCRCCheck := (uppercase(GetPiece(ParamStr(i), '=', 2)) = 'YES');

      if InString(ParamStr(i), ['/brokertimeout=', '-brokertimeout=', 'brokertimeout='], False) then
        FBroker.RPCTimeLimit := StrToIntDef(GetPiece(ParamStr(i), '=', 2), 30);
    end;

  if FDebugMode then
    begin
      {
      AssignFile(f, MDRPCDebugFile);
      Rewrite(f);
      Writeln(f, 'RPC Debug Log Created: ' + DateTimeToStr(Now));
      CloseFile(f);
      MessageDlg(
        'Debug Log File created.' + #13 + #13 +
        'Filename: ' + MDRPCDebugFile + #13 + #13 +
        'Check this file for a complete log of' + #13 +
        'all remote procedure calls and results.',
         mtInformation, [mbok], 0);
      }
    end;

  if (not FDemoMode) and UseServerList then
    if GetServerInfo(SLServer, SLPort) <> 1 then
      begin
        MessageDlg('Sign-On Cancelled while selecting server', mtInformation, [mbok], 0);
        Result := False;
        Exit;
      end
    else
      begin
        FBroker.Server := SLServer;
        FBroker.ListenerPort := StrToIntDef(SLPort, 9200);
      end;
  (*
  { This sets up the demo version setup }
  if FDemoMode then
    if MessageDlg(
      'This is a demo version of the' + #13 +
      ExtractFileName(Application.Exename) + ' program.' + #13 + #13 +
      'It will attempt connection to the' + #13 +
      'Hines OIFO Development server at' + #13 + #13 +
      'IP Address: 127.000.0.1' + #13 +
      'Listener Port: 9100' + #13 + #13 +
      'Do you wish to continue?', mtConfirmation, [mbYes, mbNo], 0) = mrYes then
      begin
        FBroker.Server := '127.000.0.1';
        FBroker.ListenerPort := 9100;
        FDemoMode := True;
        FByPassCRCCheck := True;
        FDebugMode := False;
        FBroker.AccessVerifyCodes := 'cpuser;cpuser';
      end
    else
      Exit;
  *)
  try
    FBroker.Connected := True;
  except
    on E: EBrokerError do
      begin
        MessageDlg(
          'Error Encountered' + #13 + #13 +
          'User Sign-on not complete.' + #13 +
          'Attempted connection using the following:' + #13 +
          'VistA Server: ' + FBroker.Server + #13 +
          'Listener Port: ' + IntToStr(FBroker.ListenerPort) + #13 +
          'Error Message: ' + #13 + E.Message,
          mtError,
          [mbok],
          0);
        Result := False;
        Exit;
      end;
  else
    raise;
    Result := False;
  end;

  if not FBroker.CreateContext(Context) then
    begin
      Result := False;
      Exit;
    end;

  if not OnLine then
    Exit;

  with MDVersionInformation do
    if GetParameterValue(xparVersionChk, peSystem, FileNameAndVersion) <> 'YES' then
      begin
        MessageDlg(
          'This version ' + FileNameAndVersion + ' is not listed as' + #13 +
          'compatible with the current server version of Clinical Procedures',
          mtError, [mbok], 0);
        if Uppercase(ExtractFileName(FileName)) <> 'CPMANAGER.EXE' then
          begin
            Result := False;
            Exit;
          end;
      end;

  CallVistaServer(FBroker, RPC_TMDUSER, ['SIGNON', ''], nil);

  with FBroker do
    begin
      FDUZ := GetPiece(Results[0], '^', 1);
      FSignOnDt := Now;
      FName := Results[1];
      FDomainIEN := Results[2];
      FDomain := Results[3];
      FDivisionIEN := Results[4];
      FDivision := Results[5];
      FCPManager := BOOLEAN01[StrToIntDef(Results[6], 0)];
      FTitle := Results[7];
      FDTime := StrToIntDef(Results[9], 300);
      FUCI := GetPiece(Results[10], '^', 1);
      FVolume := GetPiece(Results[10], '^', 2);
      FNode := GetPiece(Results[10], '^', 3);
    end;

  {Final check to determine if the user is running the proper executable}
  if GetParameterValue(xparCrcBypass, peSystem) <> 'NO' then
    Result := True
  else
    begin
      CRCExpected := GetParameterValue(xparCrcValues, peSystem, MDVersionInformation.FileNameAndVersion);
      CRCActual := MDVersionInformation.CRCValue;
      if (CRCExpected <> CRCActual) and
        (not FByPassCRCCheck) then
        Result := (MessageDlg(
          'This program is not the recommended version' + #13 +
          'Using this version may cause problems.' + #13 + #13 +
          'Parameter:  ' + #9 + MDPARAMETERNAME[xparCrcValues] + #13 +
          'Application:' + #9 + MDVersionInformation.FileNameAndVersion + #13 +
          'Expected CRC:' + #9 + CRCExpected + #13 +
          'Computed CRC:' + #9 + CRCActual + #13 + #13 +
          'Do you wish to continue?',
          mtWarning, [mbYes, mbNo], 0) = mrYes)
      else
        Result := True;
    end;
  Application.HelpFile := IncludeTrailingPathDelimiter(FHelpFileDirectory) + ChangeFileExt(ExtractFileName(Application.ExeName), '.chm');
  LoadHelpFile(Application.HelpFile);
end;

function TMDUser.CreateContext(Context: string): Boolean;
begin
  Result := FBroker.CreateContext(Context);
end;

function TMDUser.SignOff: Boolean;
//var
//  f: TextFile;
begin
  {
  if FDebugMode then
    begin
            AssignFile(f, MDRPCDebugFile);
            Append(f);
            Writeln(f, '');
            Writeln(f, '==========================================================');
            Writeln(f, 'RPC Debug Log Closed: ' + DateTimeToStr(Now));
            Writeln(f, '==========================================================');
            CloseFile(f);

    end;
  }
  Result := True;
end;

function TMDUser.GetUserParameter(ASetting: TMDUserSetting): string;
begin
  Result := GetParameterValue(xparUserSettings, peUser, MDUSERSETTING[ASetting]);
end;

procedure TMDUser.SetUserParameter(ASetting: TMDUserSetting; const Value: string);
begin
  SetParameterValue(Value, xparUserSettings, peUser, MDUSERSETTING[ASetting]);
end;

function TMDUser.CheckESig: Boolean;
var
  ESigForm: TForm;
  ESigAttempts: integer;
  ESigOkay: Boolean;
  ESigEdit: TEdit;
begin
  ESigForm := TForm.Create(Application);
  with ESigForm do
    begin
      Width := 230;
      Height := 120;
      Caption := 'Electronic Signature Required';
      Position := poScreenCenter;
      BorderIcons := [];
      BorderStyle := bsDialog;
    end;
  with TLabel.Create(ESigForm) do
    begin
      Parent := ESigForm;
      Left := 10;
      Top := 10;
      Width := 150;
      Caption := 'Please Enter Your Electronic Signature:';
    end;
  ESigEdit := TEdit.Create(ESigForm);
  with ESigEdit do
    begin
      Parent := ESigForm;
      Text := '';
      Left := 10;
      Top := 25;
      Width := 200;
      PasswordChar := '*';
    end;
  with TButton.Create(ESigForm) do
    begin
      Parent := ESigForm;
      Left := 10;
      Top := 55;
      Width := 75;
      Caption := '&Cancel';
      ModalResult := mrCancel;
    end;
  with TButton.Create(ESigForm) do
    begin
      Parent := ESigForm;
      Left := 135;
      Top := 55;
      Width := 75;
      Caption := '&OK';
      ModalResult := mrOk;
      Default := True;
    end;
  ESigAttempts := 0;
  ESigOkay := False;

  while (ESigAttempts < 3) and (ESigOkay = False) do
    begin
      ESigEdit.Text := '';
      ESigForm.showmodal;
      if ESigForm.ModalResult = mrOk then
        begin
          Inc(ESigAttempts);
          if DemoMode then
            ESigOkay := True
          else
            begin
              CallVistaServer(FBroker, RPC_TMDUSER, ['ESIG', Encrypt(ESigEdit.Text)], nil);
              if GetPiece(FBroker.Results[0], '^', 1) = '1' then
                begin
                  ESigOkay := True;
                end
              else
                MessageDlg('Sorry, that is not your Electronic Signature Code', mtError, [mbok], 0);
            end;
        end
      else
        begin
          ESigOkay := False;
          ESigAttempts := 3;
        end;
    end;

  Result := ESigOkay;

  ESigForm.free;
end;

function TMDUser.SubmitToTaskman(RoutineReference: string): Boolean;
begin
  if CallVistaServer(Broker, RPC_TMDUSER, ['TASKMANJOB', RoutineReference], nil) then
    Result := (GetPiece(Broker.Results[0], '^', 1) <> '-1')
  else
    Result := False;
end;

function TMDRecordID.GetHeaderData(Index: integer): string;
begin
  Result := FHeaderData[Index];
end;

procedure TMDRecordID.SetHeaderData(Index: integer; const Value: string);
begin
  FHeaderData[Index] := Value;
end;

function TMDRecordID.SetFieldValue(Field, Value: string): Boolean;
var
  Ret: string;
begin
  CallVistaServer(FBroker, RPC_TMDRECORDID, ['SETFDA', DDNumber, IENS, Field, Value], nil, Ret);
  Result := (getPiece(Ret, '^', 1) <> '-1');
end;

procedure TMDRecordID.SaveChanges;
begin
  CallVistaServer(FBroker, RPC_TMDRECORDID, ['SAVEFDA', DDNumber, IENS], nil, nil);
end;

// TMDVString

constructor TMDVString.Create(Header: string);
begin
  inherited Create;
  FVString := getpiece(Header, '^', 1);
  FDateTime := FMDTToTDateTime(StrToFloat(getpiece(Header, '^', 2)));
  FLocation := getpiece(Header, '^', 3);
  FStatus := getpiece(Header, '^', 4);
  FLocationIEN := getpiece(getpiece(Header, '^', 1), ';', 3);
end;

destructor TMDVString.Destroy;
begin
  inherited Destroy;
end;

{TMDOUTPUT}

constructor TMDOutput.Create(Broker: TRPCBroker);
begin
  inherited Create;
  FBroker := Broker;
end;

destructor TMDOutput.Destroy;
begin
  inherited Destroy;
end;

function TMDOutput.Execute: Boolean;
var
  PageBorders: TRect;
  PageMargin: integer;
  PrintDate: string;
  PrintText: TStringList;
  LinePos: integer;

  procedure PrintPage(SubHeader: string);
  begin
    with Printer.Canvas do
      begin
        Font.Name := 'Arial';
        Font.Size := 12;
        Font.Style := [fsBold];
        Rectangle(PageBorders);

        {Header}
        MoveTo(PageBorders.Left, trunc(Printer.PageHeight * 0.08));
        LineTo(PageBorders.Right, trunc(Printer.PageHeight * 0.08));
        TextOut(PageBorders.Left + PageMargin,
          trunc(Printer.PageHeight * 0.045) - TextHeight('X'), Title);
        Font.Size := 10;
        Font.Style := [];
        TextOut(PageBorders.Right - TextWidth(PrintDate) - PageMargin,
          trunc(Printer.PageHeight * 0.045) - TextHeight('X'), PrintDate);

        {Subheader}
        if SubHeader <> '' then
          begin
            MoveTo(PageBorders.Left, trunc(Printer.PageHeight * 0.06));
            LineTo(PageBorders.Right, trunc(Printer.PageHeight * 0.06));
            Font.Style := [fsBold];
            if FCompressed then
              Font.Size := 7
            else
              Font.Size := 9;
            Font.Name := 'Courier New';
            TextOut(PageBorders.Left + PageMargin,
              trunc(Printer.PageHeight * 0.065), SubHeader);
          end;

        {Footer}
        MoveTo(PageBorders.Left, trunc(Printer.PageHeight * 0.96));
        LineTo(PageBorders.Right, trunc(Printer.PageHeight * 0.96));
        Font.Size := 8;
        Font.Style := [];
        Font.Name := 'Arial';
        TextOut(PageBorders.Left + PageMargin,
          trunc(Printer.PageHeight * 0.965), 'Clinical Procedures V1.0');
        TextOut(PageBorders.Right - TextWidth('Page: 00') - PageMargin,
          trunc(Printer.PageHeight * 0.965), 'Page: ' + IntToStr(Printer.PageNumber));

        {Get ready to print}

        LinePos := trunc(Printer.PageHeight * 0.085);
        Font.Name := 'Courier New';
        if FCompressed then
          Font.Size := 7
        else
          Font.Size := 9;
        Font.Style := [];
      end;
  end;

begin
  PrintDate := 'Printed: ' + DateTimeToStr(Now);
  PageBorders.Left := trunc(Printer.PageWidth * 0.02);
  PageBorders.Right := Printer.PageWidth - PageBorders.Left;
  PageBorders.Top := trunc(Printer.PageHeight * 0.02);
  PageBorders.Bottom := Printer.PageHeight - PageBorders.Top;
  PageMargin := trunc(Printer.PageWidth * 0.01);
  Printer.BeginDoc;
  PrintText := TStringList.Create;
  with PrintText do
    try
      CallVistaServer(Broker, RPC_TMDOUTPUT, ['EXECUTE', Routine], nil);
      PrintText.Assign(Broker.Results);
      PrintPage(PrintText[0]);
      while PrintText.Count > 1 do
        begin
          Printer.Canvas.TextOut(PageBorders.Left + PageMargin, LinePos, PrintText[1]);
          LinePos := LinePos + Printer.Canvas.TextHeight('X');
          PrintText.Delete(1);
          if (PrintText.Count > 3) and ((LinePos + (Printer.Canvas.TextHeight('X') * 4)) > PageBorders.Bottom) then
            begin
              Printer.NewPage;
              PrintPage(PrintText[0]);
            end;
        end;
    finally
      Printer.EndDoc;
      free;
      Result := True;
    end;
end;

{ TMDStudy }

constructor TMDStudy.Create(Broker: TRPCBroker);
begin
  inherited Create(Broker);
  DDNumber := '702';
end;

destructor TMDStudy.Destroy;
begin
  inherited Destroy;
end;

procedure TMDStudy.AddFile(AFileName, AResultReport: string);
var
  Ret: string;
begin
  CallVistaServer(
    MDBroker,
    RPC_TMDTRANSACTION,
    ['FILES', IEN + '^' + AResultReport + '^' + LowerCase(AFileName) + '^' + '1'],
    nil,
    Ret);
end;

procedure TMDStudy.AddICDCodes(ACodes: string);
var
  i: integer;
const
  FLDS: array[1..4] of string = ('.14', '.15', '.16', '.17');
begin
  for i := 1 to 4 do
    if GetPiece(ACodes, ';', i) <> '' then
      begin
        ShowMessageFmt(
          'Adding an icd code to the study. Field: %s, Value: %s',
          [FLDS[i], GetPiece(ACodes, ';', i)]
          );
        SetFieldValue(FLDS[i], GetPiece(ACodes, ';', i));
      end;
  SaveChanges;
end;

procedure TMDStudy.AddCPTCodes(ACodes: string);
begin
  ShowMessageFmt('Adding a cpt code to the study [%s]', [ACodes]);
  SetFieldValue('.18', ACodes);
  SaveChanges;
end;

procedure TMDStudy.AddPhysician(APhysician: string);
begin
  //
end;

procedure TMDStudy.DisplayErrors(ADisplay: TMemo);
var
  i: integer;
  RetList: TStringList;
begin
  RetList := TStringList.Create;
  ADisplay.Lines.Clear;
  try
    CallVistaServer(MDBroker, RPC_TMDTRANSACTION, ['GETERR', IEN], nil, RetList);
    for i := 1 to RetList.Count - 1 do
      ADisplay.Lines.Add(
        GetPiece(RetList[i], '^', 2) + #9 +
        GetPiece(RetList[i], '^', 3) + #9 +
        GetPiece(RetList[i], '^', 4));
  finally
    FreeAndNil(RetList);
  end;
end;

function TMDStudy.GetAutoSubmitToVistAImaging: Boolean;
begin
  Result := FieldValue('.04:.07', True) = '1';
end;

function TMDStudy.GetDateCreated: string;
var
  tmpDt: TDateTime;
begin
  try
    tmpDt := FMDTToTDateTime(StrToFloat(HeaderData[3]));
    Result := DateTimeToStr(tmpDt);
  except
    on EConvertError do
      Result := '';
  end;
end;

function TMDStudy.GetImageCount: integer;
begin
  Result := StrToIntDef(HeaderData[6], 0);
end;

procedure TMDStudy.SetImageCount(const Value: integer);
begin
  HeaderData[6] := IntToStr(Value);
end;

function TMDStudy.GetInstrument: string;
begin
  Result := HeaderData[5];
end;

function TMDStudy.GetProcedureName: string;
begin
  Result := HeaderData[2];
end;

function TMDStudy.GetSortableDateCreated: Double;
begin
  try
    Result := StrToFloat(HeaderData[3]);
  except
    on EConvertError do
      Result := 0;
  else
    raise;
  end;
end;

function TMDStudy.GetStatus: string;
begin
  Result := HeaderData[4]; //FieldValue('.09');
end;

function TMDStudy.GetTreatingSpecialityIEN: string;
begin
  Result := HeaderData[1];
end;

procedure TMDStudy.RemoveFile(AFileName: string);
var
  Ret: string;
begin
  CallVistaServer(MDBroker, RPC_TMDTRANSACTION, ['FILES', IEN + '^^' + LowerCase(AFileName) + '^' + '0'], nil, Ret);
end;

function TMDStudy.SubmitToVistaImaging(var StatusMessage: string): Boolean;
var
  Status: string;
begin
  if CallVistaServer(Broker, RPC_TMDTRANSACTION, ['SUBMIT', IENS], nil, Status) then
    Result := (GetPiece(Status, '^', 1) <> '-1')
  else
    Result := False;
  StatusMessage := GetPiece(Status, '^', 2);
end;

procedure TMDStudy.UpdateStatus(ANewStatus: TMDStudyStatus);
begin
  CallVistaServer(MDBroker, RPC_TMDTRANSACTION, ['NEWSTAT', IENS, IntToStr(integer(ANewStatus))], nil, nil);
  HeaderData[4] := MDSTUDYSTATUS[ANewStatus];
end;

{ TMDPatient }

constructor TMDPatient.Create(Broker: TRPCBroker);
begin
  inherited Create(Broker);
  FDDNumber := '2';
  FStudies := TStringList.Create;
end;

destructor TMDPatient.Destroy;
begin
  ClearStudyList;
  FreeAndNil(FStudies);
  inherited Destroy;
end;

function TMDPatient.GetStudies(Index: integer): TMDStudy;
begin
  Result := TMDStudy(FStudies.Objects[Index]);
end;

function TMDPatient.GetStudyCount: integer;
begin
  Result := FStudies.Count;
end;

function TMDPatient.NewStudy: TMDStudy;
begin
  Result := nil;
end;

procedure TMDPatient.ClearStudyList;
begin
  while FStudies.Count > 0 do
    begin
      TMDStudy(FStudies.Objects[0]).Free;
      FStudies.Delete(0);
    end;
end;

procedure TMDPatient.LoadStudies;
var
  i: integer;
begin
  ClearStudyList;
  CallVistaServer(MDBroker, RPC_TMDPATIENT, ['GETTRAN', DFN], nil, FStudies);
  FStudies.Delete(0);
  for i := 0 to FStudies.Count - 1 do
    FStudies.Objects[i] := TMDStudy.CreateFromHeader(MDBroker, FStudies[i]);
end;

procedure TMDPatient.SetDFN(const Value: string);
begin
  ClearStudyList;
  IEN := Value;
  FPatientName := '';
  FSSN := '';
  FDOB := '';
  FAge := '';
  FSex := '';
  FWardRoom := '';

  if Value <> '' then
    begin
      CallVistaServer(FBroker, RPC_TMDPATIENT, ['GETOBJ', Value], nil, nil);
      FPatientName := FBroker.Results[1];
      FSSN := FBroker.Results[2];
      FDOB := FBroker.Results[3];
      FAge := FBroker.Results[4];
      FSex := FBroker.Results[5];
      if FBroker.Results[6] <> '' then
        FWardRoom := FBroker.Results[6];
      LoadStudies;
    end;
end;

function TMDPatient.GetDFN: string;
begin
  Result := IEN;
end;

function TMDPatient.GetDemographics: string;
begin
  if IEN <> '' then
    Result :=
      FPatientName + '  ' +
      FWardRoom + #13 +
      FSSN + '  ' + FSex + ' ' + FDOB + ' (' + FAge + ')'
  else
    Result := 'No Patient Selected';
end;

{ TMDConsultRequest }

function TMDConsultRequest.GetCheckInCount: integer;
begin
  Result := StrToIntDef(HeaderData[8], 0);
end;

function TMDConsultRequest.GetCPDefIEN: string;
begin
  Result := HeaderData[5];
end;

function TMDConsultRequest.GetCPDefName: string;
begin
  Result := HeaderData[1];
end;

function TMDConsultRequest.GetDateTimeOrdered: string;
begin
  try
    Result := DateTimeToStr(FMDTToTDateTime(StrToFloat(HeaderData[4])));
  except
    on EConvertError do
      Result := '';
  else
    raise;
  end;
end;

function TMDConsultRequest.GetSortableDateTimeOrdered: Double;
begin
  try
    Result := StrToFloat(HeaderData[4]);
  except
    on EConvertError do
      Result := 0;
  else
    raise;
  end;
end;

function TMDConsultRequest.GetStatus: string;
begin
  Result := HeaderData[3];
end;

function TMDConsultRequest.GetUrgency: string;
begin
  Result := HeaderData[2];
end;

function TMDUser.GetUsingSharedBroker: Boolean;
begin
  Result := (FBroker is TRPCBroker);//TSharedRPCBroker);
end;

end.

