unit uVerInfo;
{
================================================================================
*
*       Application:  Clinical Procedures
*       Revision:     $Revision: 1 $  $Modtime: 8/24/04 11:06a $
*       Developer:    doma.user@domain.ext
*       Site:         Hines OIFO
*
*       Description:  Generic Version Information Object
*
*       Notes:
*
*
================================================================================
*       $Archive: /CP Version 1.0 - Patch 4 (Misc fixes)/Common/uVerInfo.pas $
*
* $History: uVerInfo.pas $
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 11/13/04   Time: 11:21a
 * Created in $/CP Version 1.0 - Patch 4 (Misc fixes)/Common
 * 
 * *****************  Version 2  *****************
 * User: Zzzzzzpetitd Date: 10/06/03   Time: 10:57a
 * Updated in $/CP Version 1.0/Common
 * Final check-in for SQA version t22
 * 
 * *****************  Version 1  *****************
 * User: Zzzzzzpetitd Date: 12/10/02   Time: 2:41p
 * Created in $/CP Version 1.0 Patch 1/Common
 * Version t13 as released
 *
 *
*
================================================================================
}
interface

uses SysUtils,
  WinTypes,
  Dialogs,
  Classes;

type
  { exception classes for version info }
  EVerInfoError = class(Exception);
  ENoVerInfoError = class(Exception);
  ENoFixeVerInfo = class(Exception);

  { different types of pre-defined version info }
  TVerInfoType =
    (viCompanyName,
    viFileDescription,
    viFileVersion,
    viInternalName,
    viLegalCopyright,
    viLegalTrademarks,
    viOriginalFilename,
    viProductName,
    viProductVersion,
    viComments);

const
  {pre-defined version information keys}
  VerNameArray: array[viCompanyName..viComments] of string[20] = (
    'CompanyName',
    'FileDescription',
    'FileVersion',
    'InternalName',
    'LegalCopyright',
    'LegalTrademarks',
    'OriginalFilename',
    'ProductName',
    'ProductVersion',
    'Comments');

type
  TCrc32Table = array[0..255] of Int64;
  TBuffer = array[0..1048576] of byte;

  TVerInfoRes = class
  private
    Handle: DWord;
    Size: integer;
    RezBuffer: string;
    TransTable: PLongint;
    FixedFileInfoBuf: PVSFixedFileInfo;
    FFileFlags: TStringList;
    FFileName: string;
    FCompiledDateTime: string;
    FNameSpace: string;
    FVersionMajor: integer;
    FVersionMinor: integer;
    FVersionPatch: integer;
    FVersionTest: integer;
    FCRCValue: string;
    procedure FillFixedFileInfoBuf;
    procedure FillFileVersionInfo;
    procedure FillFileMaskInfo;
    function GetComments: string;
    function GetCompanyName: string;
    function GetDebugBuild: Boolean;
    function GetFileDescription: string;
    function GetInternalName: string;
    function GetLegalCopyright: string;
    function GetLegalTrademarks: string;
    function GetOriginalFileName: string;
    function GetPreRelease: Boolean;
    function GetPrivateBuild: Boolean;
    function GetProductName: string;
    function GetSpecialBuild: Boolean;
    function GetPreDefKeyString(AVerKind: TVerInfoType): string;
    function GetFileFlags: string;
    function GetVAPatchNumber: string;
    function GetVAVersionNumber: string;
    function GetVAReleaseDate: string;
    function GetFileNameAndVersion: string;
  protected
    function GetFileVersion: string;
    function GetProductVersion: string;
    function GetFileOS: string;
    function VersionString: string;
  public
    constructor Create(AFileName: string; ANameSpace: string = '');
    destructor Destroy; override;
    function GetUserDefinedKey(AKey: string): string;
    property Comments: string
      read GetComments;
    property CompanyName: string
      read GetCompanyName;
    property CompiledDateTime: string
      read FCompiledDateTime;
    property CRCValue: string
      read FCRCValue;
    property FileDescription: string
      read GetFileDescription;
    property FileFlags: string
      read GetFileFlags;
    property FileName: string
      read FFileName;
    property FileNameAndVersion: string
      read GetFileNameAndVersion;
    property FileOS: string
      read GetFileOS;
    property FileVersion: string
      read GetFileVersion;
    property InternalName: string
      read GetInternalName;
    property LegalCopyright: string
      read GetLegalCopyright;
    property LegalTrademarks: string
      read GetLegalTrademarks;
    property OriginalFilename: string
      read GetOriginalFileName;
    property ProductName: string
      read GetProductName;
    property ProductVersion: string
      read GetProductVersion;
    property PreRelease: Boolean
      read GetPreRelease;
    property PrivateBuild: Boolean
      read GetPrivateBuild;
    property SpecialBuild: Boolean
      read GetSpecialBuild;
    property DebugBuild: Boolean
      read GetDebugBuild;
    property VAVersionNumber: string
      read GetVAVersionNumber;
    property VAPatchNumber: string
      read GetVAPatchNumber;
    property VAReleaseDate: string
      read GetVAReleaseDate;
  end;

function GetFileCrc32(FileName: string): string;
function UpdateCrc32(Value: integer; var Buffer: array of byte; Count: integer): integer;

implementation
{$WARN UNSAFE_CAST OFF}

uses Windows;

const
  // strings that must be fed to VerQueryValue() function
  SFInfo = '\StringFileInfo\';
  VerTranslation: PChar = '\VarFileInfo\Translation';
  FormatStr = '%s%.4x%.4x\%s%s';

  BUFFERSIZE: integer = 65536;

  CRC32TABLE: TCrc32Table =
  ($0, $77073096, $EE0E612C, $990951BA,
    $76DC419, $706AF48F, $E963A535, $9E6495A3,
    $EDB8832, $79DCB8A4, $E0D5E91E, $97D2D988,
    $9B64C2B, $7EB17CBD, $E7B82D07, $90BF1D91,
    $1DB71064, $6AB020F2, $F3B97148, $84BE41DE,
    $1ADAD47D, $6DDDE4EB, $F4D4B551, $83D385C7,
    $136C9856, $646BA8C0, $FD62F97A, $8A65C9EC,
    $14015C4F, $63066CD9, $FA0F3D63, $8D080DF5,
    $3B6E20C8, $4C69105E, $D56041E4, $A2677172,
    $3C03E4D1, $4B04D447, $D20D85FD, $A50AB56B,
    $35B5A8FA, $42B2986C, $DBBBC9D6, $ACBCF940,
    $32D86CE3, $45DF5C75, $DCD60DCF, $ABD13D59,
    $26D930AC, $51DE003A, $C8D75180, $BFD06116,
    $21B4F4B5, $56B3C423, $CFBA9599, $B8BDA50F,
    $2802B89E, $5F058808, $C60CD9B2, $B10BE924,
    $2F6F7C87, $58684C11, $C1611DAB, $B6662D3D,
    $76DC4190, $1DB7106, $98D220BC, $EFD5102A,
    $71B18589, $6B6B51F, $9FBFE4A5, $E8B8D433,
    $7807C9A2, $F00F934, $9609A88E, $E10E9818,
    $7F6A0DBB, $86D3D2D, $91646C97, $E6635C01,
    $6B6B51F4, $1C6C6162, $856530D8, $F262004E,
    $6C0695ED, $1B01A57B, $8208F4C1, $F50FC457,
    $65B0D9C6, $12B7E950, $8BBEB8EA, $FCB9887C,
    $62DD1DDF, $15DA2D49, $8CD37CF3, $FBD44C65,
    $4DB26158, $3AB551CE, $A3BC0074, $D4BB30E2,
    $4ADFA541, $3DD895D7, $A4D1C46D, $D3D6F4FB,
    $4369E96A, $346ED9FC, $AD678846, $DA60B8D0,
    $44042D73, $33031DE5, $AA0A4C5F, $DD0D7CC9,
    $5005713C, $270241AA, $BE0B1010, $C90C2086,
    $5768B525, $206F85B3, $B966D409, $CE61E49F,
    $5EDEF90E, $29D9C998, $B0D09822, $C7D7A8B4,
    $59B33D17, $2EB40D81, $B7BD5C3B, $C0BA6CAD,
    $EDB88320, $9ABFB3B6, $3B6E20C, $74B1D29A,
    $EAD54739, $9DD277AF, $4DB2615, $73DC1683,
    $E3630B12, $94643B84, $D6D6A3E, $7A6A5AA8,
    $E40ECF0B, $9309FF9D, $A00AE27, $7D079EB1,
    $F00F9344, $8708A3D2, $1E01F268, $6906C2FE,
    $F762575D, $806567CB, $196C3671, $6E6B06E7,
    $FED41B76, $89D32BE0, $10DA7A5A, $67DD4ACC,
    $F9B9DF6F, $8EBEEFF9, $17B7BE43, $60B08ED5,
    $D6D6A3E8, $A1D1937E, $38D8C2C4, $4FDFF252,
    $D1BB67F1, $A6BC5767, $3FB506DD, $48B2364B,
    $D80D2BDA, $AF0A1B4C, $36034AF6, $41047A60,
    $DF60EFC3, $A867DF55, $316E8EEF, $4669BE79,
    $CB61B38C, $BC66831A, $256FD2A0, $5268E236,
    $CC0C7795, $BB0B4703, $220216B9, $5505262F,
    $C5BA3BBE, $B2BD0B28, $2BB45A92, $5CB36A04,
    $C2D7FFA7, $B5D0CF31, $2CD99E8B, $5BDEAE1D,
    $9B64C2B0, $EC63F226, $756AA39C, $26D930A,
    $9C0906A9, $EB0E363F, $72076785, $5005713,
    $95BF4A82, $E2B87A14, $7BB12BAE, $CB61B38,
    $92D28E9B, $E5D5BE0D, $7CDCEFB7, $BDBDF21,
    $86D3D2D4, $F1D4E242, $68DDB3F8, $1FDA836E,
    $81BE16CD, $F6B9265B, $6FB077E1, $18B74777,
    $88085AE6, $FF0F6A70, $66063BCA, $11010B5C,
    $8F659EFF, $F862AE69, $616BFFD3, $166CCF45,
    $A00AE278, $D70DD2EE, $4E048354, $3903B3C2,
    $A7672661, $D06016F7, $4969474D, $3E6E77DB,
    $AED16A4A, $D9D65ADC, $40DF0B66, $37D83BF0,
    $A9BCAE53, $DEBB9EC5, $47B2CF7F, $30B5FFE9,
    $BDBDF21C, $CABAC28A, $53B39330, $24B4A3A6,
    $BAD03605, $CDD70693, $54DE5729, $23D967BF,
    $B3667A2E, $C4614AB8, $5D681B02, $2A6F2B94,
    $B40BBE37, $C30C8EA1, $5A05DF1B, $2D02EF8D);

constructor TVerInfoRes.Create(AFileName: string; ANameSpace: string = '');
begin
  FFileName := aFileName;
  FNameSpace := aNameSpace;
  FFileFlags := TStringList.Create;
  FCompiledDateTime := FormatDateTime('mmmm d, yyyy', FileDateToDateTime(FileAge(FFileName)));
  //FCompiledDateTime := DateTimeToStr(FileDateToDateTime(FileAge(FFileName)));
  FillFileVersionInfo;
  FillFixedFileInfoBuf;
  FillFileMaskInfo;
  with FixedFileInfoBuf^ do
    begin
      FVersionMajor := HIWORD(dwFileVersionMS);
      FVersionMinor := LOWORD(dwFileVersionMS);
      FVersionPatch := HIWORD(dwFileVersionLS);
      FVersionTest := LOWORD(dwFileVersionLS);
    end;
  FCRCValue := GetFileCrc32(AFileName);
end;

destructor TVerInfoRes.Destroy;
begin
  FFileFlags.Free;
end;

procedure TVerInfoRes.FillFileVersionInfo;
var
  SBSize: UInt;
begin
  // Determine size of version information
  Size := GetFileVersionInfoSize(PChar(FFileName), Handle);
  if Size <= 0 then { raise exception if size <= 0 }
    raise ENoVerInfoError.Create('No Version Info Available.');

  // Set the length accordingly
  SetLength(RezBuffer, Size);
  // Fill the buffer with version information, raise exception on error
  if not GetFileVersionInfo(PChar(FFileName), Handle, Size, PChar(RezBuffer)) then
    raise EVerInfoError.Create('Cannot obtain version info.');

  // Get translation info, raise exception on error
  if not VerQueryValue(PChar(RezBuffer), VerTranslation, pointer(TransTable),
    SBSize) then
    raise EVerInfoError.Create('No language info.');
end;

procedure TVerInfoRes.FillFixedFileInfoBuf;
var
  Size: Cardinal;
begin
  if VerQueryValue(PChar(RezBuffer), '\', Pointer(FixedFileInfoBuf), Size) then
    begin
      if Size < SizeOf(TVSFixedFileInfo) then
        raise eNoFixeVerInfo.Create('No fixed file info');
    end
  else
    raise eNoFixeVerInfo.Create('No fixed file info')
end;

procedure TVerInfoRes.FillFileMaskInfo;
begin
  with FixedFileInfoBuf^ do
    begin
      if (dwFileFlagsMask and dwFileFlags and VS_FF_PRERELEASE) <> 0 then
        FFileFlags.Add('Pre-release');
      if (dwFileFlagsMask and dwFileFlags and VS_FF_PRIVATEBUILD) <> 0 then
        FFileFlags.Add('Private build');
      if (dwFileFlagsMask and dwFileFlags and VS_FF_SPECIALBUILD) <> 0 then
        FFileFlags.Add('Special build');
      if (dwFileFlagsMask and dwFileFlags and VS_FF_DEBUG) <> 0 then
        FFileFlags.Add('Debug');
    end;
end;

function TVerInfoRes.GetPreDefKeyString(AVerKind: TVerInfoType): string;
var
  P: PChar;
  S: UInt;
begin
  Result := Format(FormatStr, [SfInfo, LoWord(TransTable^), HiWord(TransTable^),
    VerNameArray[aVerKind], #0]);
  // get and return version query info, return empty string on error
  if VerQueryValue(PChar(RezBuffer), @Result[1], Pointer(P), S) then
    Result := StrPas(P)
  else
    Result := '';
end;

function TVerInfoRes.GetUserDefinedKey(AKey: string): string;
var
  P: Pchar;
  S: UInt;
begin
  Result := Format(FormatStr, [SfInfo, LoWord(TransTable^), HiWord(TransTable^), aKey, #0]);
  // get and return version query info, return empty string on error
  if VerQueryValue(PChar(RezBuffer), @Result[1], Pointer(P), S) then
    Result := StrPas(P)
  else
    Result := '';
end;

function TVerInfoRes.VersionString: string;
begin
  Result := Format('%d.%d.%d.%d', [FVersionMajor, FVersionMinor, FVersionPatch, FVersionTest]);
end;

function TVerInfoRes.GetFileVersion: string;
begin
  Result := VersionString;
  //  with FixedFileInfoBuf^ do
  //    Result := VersionString(dwFileVersionMS, dwFileVersionLS);
end;

function TVerInfoRes.GetProductVersion: string;
begin
  Result := VersionString;
  //  with FixedFileInfoBuf^ do
  //    Result := VersionString(dwProductVersionMS, dwProductVersionLS);
end;

function TVerInfoRes.GetFileOS: string;
begin
  with FixedFileInfoBuf^ do
    case dwFileOS of
      VOS_UNKNOWN: Result := 'Unknown';
      VOS_DOS: Result := 'Designed for MS-DOS';
      VOS_OS216: Result := 'Designed for 16-bit OS/2';
      VOS_OS232: Result := 'Designed for 32-bit OS/2';
      VOS_NT: Result := 'Designed for Windows NT';

      VOS__WINDOWS16: Result := 'Designed for 16-bit Windows';
      VOS__PM16: Result := 'Designed for 16-bit PM';
      VOS__PM32: Result := 'Designed for 32-bit PM';
      VOS__WINDOWS32: Result := 'Designed for 32-bit Windows';

      VOS_DOS_WINDOWS16: Result := 'Designed for 16-bit Windows, running on MS-DOS';
      VOS_DOS_WINDOWS32: Result := 'Designed for Win32 API, running on MS-DOS';
      VOS_OS216_PM16: Result := 'Designed for 16-bit PM, running on 16-bit OS/2';
      VOS_OS232_PM32: Result := 'Designed for 32-bit PM, running on 32-bit OS/2';
      VOS_NT_WINDOWS32: Result := 'Designed for Win32 API, running on Windows/NT';
    else
      Result := 'Unknown';
    end;
end;

{File CRC Functions}

function GetFileCrc32(FileName: string): string;
var
  Buffer: Pointer;
  Handle, Loaded: integer;
  TempResult: DWord;
begin
  TempResult := $FFFFFFFF;
  GetMem(Buffer, BUFFERSIZE);
  Handle := FileOpen(FileName, fmShareDenyWrite);
  repeat
    Loaded := FileRead(Handle, Buffer^, BUFFERSIZE);
    TempResult := UpdateCrc32(TempResult, TBuffer(Buffer^), Loaded);
  until Loaded <> BUFFERSIZE;
  FileClose(Handle);
  FreeMem(Buffer);
  TempResult := not TempResult;
  Result := IntToHex(TempResult, 8);
end;

function UpdateCrc32(Value: integer; var Buffer: array of byte; Count: integer): integer;
var
  i: integer;
begin
  Result := Value;
  for i := 0 to Count - 1 do
    begin
      Result := ((Result shr 8) and $00FFFFFF) xor CRC32TABLE[(Result xor Buffer[i]) and $000000FF];
    end;
end;

function TVerInfoRes.GetComments: string;
begin
  Result := GetPreDefKeyString(viComments);
end;

function TVerInfoRes.GetCompanyName: string;
begin
  Result := GetPreDefKeyString(viCompanyName);
end;

function TVerInfoRes.GetDebugBuild: Boolean;
begin
  Result := (FFileFlags.IndexOf('Debug') > -1);
end;

function TVerInfoRes.GetFileDescription: string;
begin
  Result := GetPreDefKeyString(viFileDescription);
end;

function TVerInfoRes.GetInternalName: string;
begin
  Result := GetPreDefKeyString(viInternalName);
end;

function TVerInfoRes.GetLegalCopyright: string;
begin
  Result := GetPreDefKeyString(viLegalCopyright);
end;

function TVerInfoRes.GetLegalTrademarks: string;
begin
  Result := GetPreDefKeyString(viLegalTrademarks);
end;

function TVerInfoRes.GetOriginalFileName: string;
begin
  Result := GetPreDefKeyString(viOriginalFilename);
end;

function TVerInfoRes.GetPreRelease: Boolean;
begin
  Result := (FFileFlags.IndexOf('Pre-release') > -1);
end;

function TVerInfoRes.GetPrivateBuild: Boolean;
begin
  Result := (FFileFlags.IndexOf('Private build') > -1);
end;

function TVerInfoRes.GetProductName: string;
begin
  Result := GetPreDefKeyString(viProductName);
end;

function TVerInfoRes.GetSpecialBuild: Boolean;
begin
  Result := (FFileFlags.IndexOf('Special build') > -1);
end;

function TVerInfoRes.GetFileFlags: string;
begin
  Result := FFileFlags.CommaText;
end;

function TVerInfoRes.GetVAPatchNumber: string;
begin
  if FVersionPatch = 0 then
    Result := ''
  else
    begin
      Result :=
        GetUserDefinedKey('VANamespace') + '*' +
        IntToStr(FVersionMajor) + '.' + IntToStr(FVersionMinor) + '*' +
        IntToStr(FVersionPatch);
      if FVersionTest > 0 then
        Result := Result + 'T' + IntToStr(FVersionTest);
    end;
end;

function TVerInfoRes.GetVAVersionNumber: string;
begin
  Result := IntToStr(FVersionMajor) + '.' + IntToStr(FVersionMinor);
  if (FVersionTest > 0) and (FVersionPatch = 0) then
    Result := Result + 'T' + IntToStr(FVersionTest);
end;

function TVerInfoRes.GetVAReleaseDate: string;
begin
  Result := GetUserDefinedKey('VAReleaseDate');
end;

function TVerInfoRes.GetFileNameAndVersion: string;
begin
  Result := UpperCase(ExtractFileName(FFileName)) + ':' + VersionString;
end;

end.

